package com.starbuds.server.service.user.impl;

import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.AgentAuditDO;
import com.starbuds.server.common.pojo.user.AgentAuditVO;
import com.starbuds.server.common.pojo.user.AgentVO;
import com.starbuds.server.common.pojo.user.AnchorSignedRatio;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.user.AgentService;
import com.starbuds.server.service.user.dao.*;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/7 16:29
 * @Description 公会业务实现
 */
@Service(version = "1.0.0")
public class AgentServiceImpl implements AgentService {

    @Autowired
    AgentDAO agentDAO;

    @Autowired
    AgentAuditDAO agentAuditDAO;

    @Autowired
    AgentAnchorDAO agentAnchorDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    AnchorDAO anchorDAO;


    /**
     * 公会是否存在
     *
     * @param agentId 公会id
     * @return 是否存在
     */
    @Override
    public boolean isAgentExist(Long agentId) {
        return agentDAO.countById(agentId) > 0;
    }

    /**
     * 获取公会列表
     *
     * @param content  内容
     * @param pageNum  当前页码
     * @param pageSize 分页大小
     * @return 分页对象
     */
    @Override
    public PageList<AgentVO> getAgentPageList(String content, int pageNum, int pageSize) {
        PageList<AgentVO> page = new PageList<>(pageNum, pageSize);
        page.setList(agentDAO.getAgentPageList(content, page.getPageCursor(), pageSize));
        page.setTotal(agentDAO.countAgentPageList(content));
        return page;
    }

    /**
     * 获取公会详情
     *
     * @param agentId 公会id
     * @return 公会详情
     */
    @Override
    public AgentVO getAgentDetail(Long agentId) {
        AgentVO vo = agentDAO.getAgentDetail(agentId);
        if (vo != null) {
            vo.setAnchorCount(agentAnchorDAO.getAnchorCount(agentId));
        }
        return vo;
    }

    /**
     * 申请签约
     *
     * @param agentId  公会id
     * @param userId   用户id
     * @param applyMsg 申请时备注
     * @return 是否成功
     */
    @Override
    public boolean applySign(Long agentId, Long userId, String applyMsg) {
        // 发起申请
        AgentAuditDO auditDO = new AgentAuditDO();
        auditDO.setAuditId(identityService.getId());
        auditDO.setAgentId(agentId);
        auditDO.setUserId(userId);
        auditDO.setApplyMsg(applyMsg);
        auditDO.setApplyTime(System.currentTimeMillis());
        auditDO.setAuditStatus(AuditStatusEnum.WAIT.getCode());
        auditDO.setApplyType(ApplyTypeEnum.JOIN.getCode());

        return agentAuditDAO.insert(auditDO) == 1;
    }

    /**
     * 申请解约
     *
     * @param agentId  公会id
     * @param userId   用户id
     * @param applyMsg 申请时备注
     * @return 是否成功
     */
    @Override
    public boolean cancelSign(Long userId, Long agentId, String applyMsg) {
        //发起申请
        AgentAuditDO auditDO = new AgentAuditDO();
        auditDO.setAuditId(identityService.getId());
        auditDO.setAgentId(agentId);
        auditDO.setUserId(userId);
        auditDO.setApplyMsg(applyMsg);
        auditDO.setApplyTime(System.currentTimeMillis());
        auditDO.setAuditStatus(AuditStatusEnum.WAIT.getCode());
        auditDO.setApplyType(ApplyTypeEnum.LEAVE.getCode());

        return agentAuditDAO.insert(auditDO) == 1;
    }

    /**
     * 是否签约指定公会
     *
     * @param agentId 公会id
     * @param userId  用户id
     * @return 是否签约
     */
    @Override
    public boolean isSign(Long agentId, Long userId) {
        return agentAnchorDAO.countByAgentIdAndUserId(agentId, userId) > 0;
    }

    /**
     * 查询审核状态
     *
     * @param userId 用户id
     * @return 签约状态
     */
    @Override
    public Integer getAuditStatus(Long userId) {
        return agentAuditDAO.getAuditStatus(userId);
    }

    /**
     * 获取公会id  不判断状态
     *
     * @param userId
     * @return
     */
    @Override
    public Long getAgentId(Long userId) {
        return agentAuditDAO.getAgentId(userId, ApplyTypeEnum.JOIN.getCode());
    }

    /**
     * 获取用户最近一次申请信息
     *
     * @param userId    用户id
     * @param applyType 申请类型
     * @return 申请信息
     */
    @Override
    public AgentAuditVO getLastApplyInfo(Long userId, Integer applyType) {
        AgentAuditVO vo = agentAuditDAO.getLastApplyInfo(userId, applyType);
        if (vo != null && ApplyTypeEnum.LEAVE.getCode() == vo.getApplyType()) {
            vo.setAgentId(anchorDAO.getAgentId(userId));
        }
        return vo;
    }

    /**
     * 读取主播签约工会以及提成比例
     *
     * @param userId
     * @return
     */
    @Override
    public AnchorSignedRatio getAnchorSignedRatio(Long userId) {
        Long agentId = getAgentId(userId);
        if (agentId != null && agentId > 0) {
            return agentAnchorDAO.selectSignedRatio(userId, agentId);
        }
        return null;
    }
}
