package com.starbuds.server.service.user.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.api.HeaderPackage;
import com.starbuds.server.common.pojo.db.DeviceDO;
import com.starbuds.server.common.pojo.mq.QueueDeviceAddMsg;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.DeviceService;
import com.starbuds.server.service.user.dao.DeviceDAO;
import com.starbuds.server.service.user.dao.UserLoginHisDAO;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * 设备服务
 *
 * @author Lay
 * @date 2020/2/7
 */
@Service(version = "1.0.0")
@Slf4j
public class DeviceServiceImpl implements DeviceService {

    @Autowired
    DeviceDAO deviceDAO;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    UserLoginHisDAO userLoginHisDAO;

    /**
     * 添加设备
     *
     * @param deviceDO
     * @return
     */
    @Override
    public boolean addDevice(DeviceDO deviceDO) {

        QueueDeviceAddMsg deviceAddMsg = new QueueDeviceAddMsg();
        deviceAddMsg.setChannelId(deviceDO.getChannelId());
        deviceAddMsg.setIdfa(deviceDO.getIdfa());
        deviceAddMsg.setImeiMd5(deviceDO.getImeiMd5());

        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.DEVICE_CONNECT, deviceAddMsg.serialize());

        if (deviceDAO.countById(deviceDO.getDeviceId()) > 0) {
            return false;
        }

        try {
            deviceDO.setDeviceLocked(BoolIntVal.False);
            deviceDO.setDeviceCreateTime(System.currentTimeMillis());
            deviceDAO.insert(deviceDO);

            ClientType clientType = ClientType.toEnum(deviceDO.getClientType());
            if (ClientType.iOS == clientType || ClientType.Android == clientType) {
                statPlatformService.addPlatformStat(deviceDO.getChannelId(), StatPlatformTypeEnum.INSTALL_COUNT, BigDecimal.ONE);
                switch (clientType) {
                    case iOS:
                        statPlatformService.addPlatformStat(deviceDO.getChannelId(), StatPlatformTypeEnum.INSTALL_COUNT_IOS, BigDecimal.ONE);
                        break;
                    case Android:
                        statPlatformService.addPlatformStat(deviceDO.getChannelId(), StatPlatformTypeEnum.INSTALL_COUNT_ANDROID, BigDecimal.ONE);
                        break;
                    default:
                        break;
                }
            }

            return true;
        } catch (Exception e) {
            log.error("设备创建失败", e);
            return false;
        }
    }

    /**
     * 根据设备 id 查询
     *
     * @param deviceId
     * @return
     */
    @Override
    public DeviceDO getDeviceById(String deviceId) {
        return deviceDAO.selectById(deviceId);
    }

    /**
     * 设置设备锁定状态
     *
     * @param deviceId
     * @param locked
     * @return
     */
    @Override
    public boolean setDeviceLocked(String deviceId, Integer locked) {
        try {
            return deviceDAO.updateLocked(deviceId, locked) > 0;
        } catch (Exception e) {
            log.error("设备锁定状态设置失败", e);
        }
        return false;
    }

    /**
     * 判断设备是否被锁定
     *
     * @param deviceId
     * @return
     */
    @Override
    public boolean isLocked(String deviceId) {
        DeviceDO deviceDO = getDeviceById(deviceId);
        if (deviceDO != null) {
            if (BoolIntVal.True == deviceDO.getDeviceLocked()) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断设备是否存在
     *
     * @param deviceId
     * @return
     */
    @Override
    public boolean isExists(String deviceId) {
        return deviceDAO.countById(deviceId) > 0;
    }

    /**
     * 获取设备id
     *
     * @param userId
     * @return
     */
    @Override
    public String getDeviceId(Long userId) {
        String headers = userLoginHisDAO.selectLastHeaders(userId);
        if (StringUtils.isNotBlank(headers)) {
            HeaderPackage headerPackage = JSON.parseObject(headers, HeaderPackage.class);
            if (headerPackage != null) {
                return headerPackage.getDeviceId();
            }
        }
        return null;
    }
}
