package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.RandomUtil;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.NiceNoHisDO;
import com.starbuds.server.common.pojo.db.NiceNoPoolDO;
import com.starbuds.server.common.pojo.db.UserBagDO;
import com.starbuds.server.common.pojo.db.UserBagHisDO;
import com.starbuds.server.common.pojo.mq.QueueNiceNoExpireMsg;
import com.starbuds.server.common.pojo.user.NiceNoVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.user.NiceNoService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.LiveUserProvider;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 靓号服务实现
 *
 * @author Lay
 * @date 2019/12/24
 */
@Service(version = "1.0.0")
@Slf4j
public class NiceNoServiceImpl implements NiceNoService {

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    NiceNoPoolDAO niceNoPoolDAO;

    @Resource
    NiceNoHisDAO niceNoHisDAO;

    @Resource
    UserDAO userDAO;

    @Resource
    UserBagDAO userBagDAO;

    @Resource
    UserBagHisDAO userBagHisDAO;

    @Resource
    LiveUserProvider liveUserProvider;

    /**
     * 绑定靓号
     *
     * @param userId
     * @param niceNo
     * @param effectiveTime
     * @param remarks
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long bindNiceNo(Long userId, Long niceNo, Long effectiveTime, String remarks) {
        // 查询靓号
        NiceNoPoolDO niceNoPoolDO = niceNoPoolDAO.selectById(niceNo);

        // 判断是否已绑定
        if (niceNoPoolDO == null || BoolIntVal.True == niceNoPoolDO.getBindStatus()) {
            return null;
        }

        // Redis 锁
        if (!redisService.setIfAbsent(RedisCacheKeys.getUsersNiceNoLockKey(niceNo), String.valueOf(userId), TimeConstants.SECOND_15_MILLS, TimeUnit.MILLISECONDS)) {
            return null;
        }

        // 没有有效时间即为永久
        Long curTime = System.currentTimeMillis();
        Long expireTime;
        if (effectiveTime == null || effectiveTime < 1) {
            expireTime = -1L;
        } else {
            expireTime = curTime + effectiveTime;
        }

        // 判断原来是否有绑定
        Long oldNiceNo = userDAO.selectNiceNo(userId);
        if (oldNiceNo != null && oldNiceNo > 0) {
            // 解绑原来的靓号
            NiceNoPoolDO oldItem = new NiceNoPoolDO();
            oldItem.setNiceNo(oldNiceNo);
            oldItem.setBindStatus(BoolIntVal.False);
            oldItem.setBindExpireTime(null);
            oldItem.setUserId(null);
            oldItem.setUpdateTime(curTime);
            niceNoPoolDAO.updateBind(oldItem);
        }

        // 更新靓号
        niceNoPoolDO.setUserId(userId);
        niceNoPoolDO.setBindStatus(BoolIntVal.True);
        niceNoPoolDO.setBindExpireTime(expireTime);
        niceNoPoolDAO.updateBind(niceNoPoolDO);

        // 写入绑定记录
        Long hisId = identityService.getId();
        NiceNoHisDO niceNoHisDO = new NiceNoHisDO();
        niceNoHisDO.setHisId(hisId);
        niceNoHisDO.setNiceNo(niceNo);
        niceNoHisDO.setUserId(userId);
        niceNoHisDO.setNiceLevel(niceNoPoolDO.getNiceLevel());
        niceNoHisDO.setHisRemarks(remarks);
        niceNoHisDO.setHisCreateTime(curTime);
        niceNoHisDO.setExpireTime(expireTime);
        niceNoHisDAO.insert(niceNoHisDO);

        // 更新用户靓号
        userDAO.updateNiceNo(userId, niceNo);

        // 更新 es
        userSearchService.setNiceNo(userId, niceNo);

        // 创建过期检查消息
        if (expireTime > 0) {
            // 超时60秒后处理
            long startDeliveryTime = expireTime > curTime + TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : expireTime;
            QueueNiceNoExpireMsg expireMsg = new QueueNiceNoExpireMsg();
            expireMsg.setNiceNo(niceNo);
            expireMsg.setUserId(userId);
            expireMsg.setExpireTime(expireTime);
            expireMsg.setHisId(hisId);
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NICE_NO_EXPIRE, expireMsg.serialize(), startDeliveryTime);
        }

        // 刷新直播间资料
        liveUserProvider.refreshLiveUserProfile(userId);

        // 释放锁
        redisService.delKey(RedisCacheKeys.getUsersNiceNoLockKey(niceNo));
        return hisId;
    }

    /**
     * 使用靓号卡绑定靓号
     *
     * @param id
     * @param userId
     * @param niceNo
     * @param effectiveTime
     * @param remarks
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean bindNiceNoByBagId(String id, Long userId, Long niceNo, Long effectiveTime, String remarks) {
        // 消耗靓号卡
        UserBagDO userBagDO = userBagDAO.selectById(id);
        if (userBagDO == null || userBagDO.getQuantity() < 1) {
            return false;
        }
        Long curTime = System.currentTimeMillis();

        Long hisId = bindNiceNo(userId, niceNo, effectiveTime, remarks);
        if (hisId != null) {
            // 更新背包
            int updated = userBagDAO.updateQuantity(id, userBagDO.getQuantity() - 1, userBagDO.getVersion(), curTime);
            if (updated == 0) {
                throw new RuntimeException("更新靓号卡数量失败");
            }

            // 写入变更记录
            UserBagHisDO userBagHisDO = new UserBagHisDO();
            userBagHisDO.setHisId(identityService.getId());
            userBagHisDO.setBagId(id);
            userBagHisDO.setItemId(userBagDO.getItemId());
            userBagHisDO.setUserId(userBagDO.getUserId());
            userBagHisDO.setItemType(userBagDO.getItemType());
            userBagHisDO.setQuantity(1);
            userBagHisDO.setCreateTime(curTime);
            userBagHisDO.setHisRemarks(String.format("兑换靓号%d", niceNo));
            userBagHisDO.setOrderId(hisId);
            userBagHisDAO.insert(userBagHisDO);

            return true;
        }

        return false;
    }

    /**
     * 获取靓号信息
     *
     * @param niceNo
     * @return
     */
    @Override
    public NiceNoPoolDO getNiceNo(Long niceNo) {
        return niceNoPoolDAO.selectById(niceNo);
    }

    /**
     * 获取靓号列表
     *
     * @param pageIndex
     * @param pageSize
     * @param niceLevel
     * @param bindStatus
     * @return
     */
    @Override
    public PageList<NiceNoVO> getNiceNoListPaged(int pageIndex, int pageSize, Integer niceLevel, Integer bindStatus) {
        PageList<NiceNoVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(niceNoPoolDAO.selectAll(pageList.getPageCursor(), pageSize, niceLevel, bindStatus));
        pageList.setTotal(niceNoPoolDAO.count(niceLevel, bindStatus));
        return pageList;
    }

    /**
     * 随机获取靓号
     *
     * @param num
     * @param niceLevel
     * @param bindStatus
     * @return
     */
    @Override
    public List<NiceNoVO> getNiceNoListRandom(int num, Integer niceLevel, Integer bindStatus) {
        int total = niceNoPoolDAO.count(niceLevel, bindStatus);
        if (total > 0) {
            int pageTotal = total / num;
            // 获取随机一页
            int pageNum = pageTotal > 1 ? RandomUtil.randomInt(1, pageTotal + 1) : Math.max(pageTotal, 1);
            PageList<NiceNoVO> pageList = new PageList<>(pageNum, num);
            List<NiceNoVO> list = niceNoPoolDAO.selectAll(pageList.getPageCursor(), num, niceLevel, bindStatus);
            if (CollectionUtil.isNotEmpty(list)) {
                return list;
            }
        }
        return Collections.EMPTY_LIST;
    }

    /**
     * 获取用户靓号
     *
     * @param userId
     * @return
     */
    @Override
    public NiceNoVO getUserNiceNo(Long userId) {
        NiceNoPoolDO niceNoPoolDO = niceNoPoolDAO.selectByUserId(userId);
        if (niceNoPoolDO != null) {
            NiceNoVO niceNo = new NiceNoVO();
            BeanUtils.copyProperties(niceNoPoolDO, niceNo);
            return niceNo;
        }
        return null;
    }

    /**
     * 靓号续期
     *
     * @param id
     * @param userId
     * @param niceNo
     * @param effectiveTime
     * @param remarks
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean renewNiceNoByBagId(String id, Long userId, Long niceNo, Long effectiveTime, String remarks) {
        NiceNoPoolDO niceNoPool = getNiceNo(niceNo);
        if (niceNoPool == null || !niceNoPool.getNiceNo().equals(niceNo) || !niceNoPool.getUserId().equals(userId) || niceNoPool.getBindExpireTime() < 0) {
            return false;
        }

        // 消耗靓号卡
        UserBagDO userBagDO = userBagDAO.selectById(id);
        if (userBagDO == null || userBagDO.getQuantity() < 1) {
            return false;
        }
        Long curTime = System.currentTimeMillis();

        // 更新背包
        int updated = userBagDAO.updateQuantity(id, userBagDO.getQuantity() - 1, userBagDO.getVersion(), curTime);
        if (updated == 0) {
            throw new RuntimeException("更新靓号卡数量失败");
        }

        // 更新靓号时长
        Long expireTime = effectiveTime > 0 ? niceNoPool.getBindExpireTime() + effectiveTime : effectiveTime;
        niceNoPool.setUpdateTime(curTime);
        niceNoPool.setBindExpireTime(expireTime);
        niceNoPoolDAO.updateBind(niceNoPool);

        // 写入绑定记录
        Long hisId = identityService.getId();
        NiceNoHisDO niceNoHisDO = new NiceNoHisDO();
        niceNoHisDO.setHisId(hisId);
        niceNoHisDO.setNiceNo(niceNo);
        niceNoHisDO.setUserId(userId);
        niceNoHisDO.setNiceLevel(niceNoPool.getNiceLevel());
        niceNoHisDO.setHisRemarks(remarks);
        niceNoHisDO.setHisCreateTime(curTime);
        niceNoHisDO.setExpireTime(expireTime);
        niceNoHisDAO.insert(niceNoHisDO);

        // 写入变更记录
        UserBagHisDO userBagHisDO = new UserBagHisDO();
        userBagHisDO.setHisId(identityService.getId());
        userBagHisDO.setBagId(id);
        userBagHisDO.setItemId(userBagDO.getItemId());
        userBagHisDO.setUserId(userBagDO.getUserId());
        userBagHisDO.setItemType(userBagDO.getItemType());
        userBagHisDO.setQuantity(1);
        userBagHisDO.setCreateTime(curTime);
        userBagHisDO.setHisRemarks(String.format("续期靓号%d", niceNo));
        userBagHisDO.setOrderId(hisId);
        userBagHisDAO.insert(userBagHisDO);

        // 创建过期检查消息
        if (expireTime > 0) {
            // 超时60秒后处理
            long startDeliveryTime = expireTime > curTime + TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : expireTime;
            QueueNiceNoExpireMsg expireMsg = new QueueNiceNoExpireMsg();
            expireMsg.setNiceNo(niceNo);
            expireMsg.setUserId(userId);
            expireMsg.setExpireTime(expireTime);
            expireMsg.setHisId(hisId);
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NICE_NO_EXPIRE, expireMsg.serialize(), startDeliveryTime);
        }

        return true;
    }
}
