package com.starbuds.server.service.user.impl;

import com.starbuds.server.common.constant.user.NobleLevelEnum;
import com.starbuds.server.common.constant.user.NoblePrivilegeEnum;
import com.starbuds.server.common.pojo.db.UserNobleDO;
import com.starbuds.server.common.service.user.NoblePrivilegeService;
import com.starbuds.server.service.user.dao.UserNobleDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.starbuds.server.common.constant.user.NobleLevelEnum.*;
import static com.starbuds.server.common.constant.user.NoblePrivilegeEnum.*;

/**
 * @author aoe
 * @date 2020/11/23
 */
@Service(version = "1.0.0")
@Slf4j
public class NoblePrivilegeServiceImpl implements NoblePrivilegeService {

    @Resource
    private UserNobleDAO userNobleDAO;

    private Map<NobleLevelEnum, List<NoblePrivilegeEnum>> privilegeMap;

    @PostConstruct
    private void init() {
        privilegeMap = new HashMap<>();
        privilegeMap.put(NANJUE, nanjuePrivileges());
        privilegeMap.put(ZIJUE, zijuePrivileges());
        privilegeMap.put(BOJUE, bojuePrivileges());
        privilegeMap.put(HOUJUE, houjuePrivileges());
        privilegeMap.put(GONGJUE, gongjuePrivileges());
        privilegeMap.put(GUOWANG, guoWangPrivileges());
        privilegeMap.put(DIHUANG, diHuangPrivileges());
    }

    @Override
    public Boolean hasPrivilege(Long userId, NoblePrivilegeEnum privilege) {
        UserNobleDO userNoble = userNobleDAO.selectById(userId, System.currentTimeMillis());
        if (userNoble == null) {
            return false;
        }
        Integer nobleLevel = userNoble.getNobleLevel();
        if (nobleLevel == null) {
            return false;
        }
        NobleLevelEnum nobleLevelEnum = NobleLevelEnum.toEnum(nobleLevel);
        if (nobleLevelEnum == null) {
            return false;
        }
        List<NoblePrivilegeEnum> list = privilegeMap.get(nobleLevelEnum);
        if (CollectionUtils.isEmpty(list)) {
            return false;
        }

        return list.stream().anyMatch(p -> p == privilege);
    }

    /**
     * 男爵特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> nanjuePrivileges() {
        return createPrivileges(NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT);
    }

    /**
     * 子爵特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> zijuePrivileges() {
        return createPrivileges(NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT);
    }

    /**
     * 伯爵特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> bojuePrivileges() {
        return createPrivileges(NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT, HIDE_FOLLOWING);
    }

    /**
     * 侯爵特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> houjuePrivileges() {
        return createPrivileges(NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT, HIDE_FOLLOWING);
    }

    /**
     * 公爵特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> gongjuePrivileges() {
        return createPrivileges(MOUNT, NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT, HIDE_FOLLOWING);
    }

    /**
     * 国王特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> guoWangPrivileges() {
        return createPrivileges(MOUNT, NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT, HIDE_FOLLOWING,
                PREVENT_SILENCE, PREVENT_KICK);
    }

    /**
     * 帝皇特权
     *
     * @return
     */
    private List<NoblePrivilegeEnum> diHuangPrivileges() {
        return createPrivileges(MOUNT, NOBLE_MEDAL, ENTRY_EFFECTS, OPENING_ANNOUNCEMENT, HIDE_FOLLOWING,
                PREVENT_SILENCE, PREVENT_KICK);
    }

    /**
     * 创建特权
     *
     * @param privileges
     * @return
     */
    private List<NoblePrivilegeEnum> createPrivileges(NoblePrivilegeEnum... privileges) {
        List<NoblePrivilegeEnum> list = new ArrayList<>();
        for (NoblePrivilegeEnum privilege : privileges) {
            list.add(privilege);
        }
        return list;
    }

}
