package com.starbuds.server.service.user.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.data.BroadcastTypeEnum;
import com.starbuds.server.common.constant.finance.NoblePayTypeEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.NobleLevelEnum;
import com.starbuds.server.common.pojo.data.PropsVO;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.BagItem;
import com.starbuds.server.common.pojo.mq.QueueNobleExpireMsg;
import com.starbuds.server.common.pojo.mq.QueueNobleNewMsg;
import com.starbuds.server.common.pojo.mq.QueueNobleRenewalMsg;
import com.starbuds.server.common.pojo.mq.QueueNobleUpgradeMsg;
import com.starbuds.server.common.pojo.user.NobleUpgradeAmount;
import com.starbuds.server.common.pojo.user.UserNobleVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.BroadcastHisService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.WalletService;
import com.starbuds.server.common.service.user.NobleService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;

/**
 * 贵族服务实现
 *
 * @author Lay
 * @date 2020/3/4
 */
@Service(version = "1.0.0")
@Slf4j
public class NobleServiceImpl implements NobleService {

    @Reference(version = "1.0.0")
    WalletService walletService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    BroadcastHisService broadcastHisService;

    @Resource
    UserNobleDAO userNobleDAO;

    @Resource
    UserNobleHisDAO userNobleHisDAO;

    @Resource
    NoblePriceConfigDAO noblePriceConfigDAO;

    @Resource
    UserBagDAO userBagDAO;

    @Resource
    UserBagHisDAO userBagHisDAO;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 获取用户贵族
     *
     * @param userId
     * @return
     */
    @Override
    public UserNobleVO getUserNoble(Long userId) {
        UserNobleDO userNoble = userNobleDAO.selectById(userId, System.currentTimeMillis());
        if (userNoble != null) {
            UserNobleVO userNobleVO = new UserNobleVO();
            BeanUtils.copyProperties(userNoble, userNobleVO);
            return userNobleVO;
        }
        return null;
    }

    /**
     * 获取用户贵族等级
     *
     * @param userId
     * @return
     */
    @Override
    public Integer getUserNobleLevel(Long userId) {
        UserNobleVO userNoble = getUserNoble(userId);
        if (userNoble != null) {
            return userNoble.getNobleLevel();
        }
        return 0;
    }

    /**
     * 根据贵族等级获取价格
     *
     * @param nobleLevel
     * @return
     */
    @Override
    public NoblePriceConfigDO getPriceConfigByLevel(Integer nobleLevel) {
        return noblePriceConfigDAO.selectByLevel(nobleLevel);
    }

    /**
     * 获取贵族等级价格
     *
     * @return
     */
    @Override
    public List<NoblePriceConfigDO> getPriceConfigList() {
        return noblePriceConfigDAO.selectAll();
    }

    /**
     * 新开贵族
     *
     * @param userId
     * @param nobleLevel
     * @param channelId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long newNoble(Long userId, int nobleLevel, Long channelId) {
        NoblePriceConfigDO noblePriceConfig = getPriceConfigByLevel(nobleLevel);
        if (noblePriceConfig == null) {
            return null;
        }
        Long hisId = identityService.getId();
        Long balance = walletService.reduceCoin(userId, noblePriceConfig.getNoblePriceType(), noblePriceConfig.getNoblePrice(), String.format("开通贵族%s", NobleLevelEnum.toEnum(nobleLevel).getDesc()), hisId);
        if (balance < 0) {
            return balance;
        }

        // 写入记录
        Long curTime = System.currentTimeMillis();
        Long duration = noblePriceConfig.getDuration();
        UserNobleHisDO userNobleHis = new UserNobleHisDO();
        userNobleHis.setHisId(hisId);
        userNobleHis.setUserId(userId);
        userNobleHis.setNobleLevel(nobleLevel);
        userNobleHis.setHisAmount(noblePriceConfig.getNoblePrice());
        userNobleHis.setHisAmountType(noblePriceConfig.getNoblePriceType());
        userNobleHis.setHisType(NoblePayTypeEnum.NEW.getValue());
        userNobleHis.setHisDuration(duration);
        userNobleHis.setHisCreateTime(curTime);
        userNobleHis.setHisTimes(1);
        userNobleHis.setHisAutoRenewal(BoolIntVal.False);
        userNobleHis.setChannelId(channelId);
        userNobleHisDAO.insert(userNobleHis);

        // 更新贵族
        Long expireTime = duration > 0 ? System.currentTimeMillis() + duration : duration;
        UserNobleDO userNoble = new UserNobleDO();
        userNoble.setUserId(userId);
        userNoble.setNobleLevel(nobleLevel);
        userNoble.setAutoRenewal(BoolIntVal.True);
        userNoble.setExpireTime(expireTime);
        userNoble.setStatus(CommonStatus.Normal);
        userNoble.setUpdateTime(curTime);
        userNoble.setChannelId(channelId);
        userNobleDAO.insertOrUpdate(userNoble);

        // 丢出消息
        QueueNobleNewMsg nobleNewMsg = new QueueNobleNewMsg();
        nobleNewMsg.setUserId(userId);
        nobleNewMsg.setNobleLevel(nobleLevel);
        nobleNewMsg.setChannelId(channelId);
        nobleNewMsg.setDuration(duration);
        nobleNewMsg.setIsFromProps(false);
        nobleNewMsg.setPropsId(0L);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.NOBLE_NEW, nobleNewMsg.serialize());

        if (expireTime > 0) {
            long deliveryStartTime = duration > TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : expireTime;

            // 丢出贵族过期消息
            QueueNobleExpireMsg nobleExpireMsg = new QueueNobleExpireMsg();
            nobleExpireMsg.setUserId(userId);
            nobleExpireMsg.setHisId(hisId);
            nobleExpireMsg.setNobleLevel(nobleLevel);
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NOBLE_EXPIRE, nobleExpireMsg.serialize(), deliveryStartTime);
        }
        broadcastHisService.insert(userId, 0L, BroadcastTypeEnum.NOBEL,
                noblePriceConfig.getNoblePrice(), noblePriceConfig.getIconUrl(), 1, 0L, Long.valueOf(nobleLevel), (getNobelNameByLevel(nobleLevel) + "贵族").intern());

        return balance;
    }

    private String getNobelNameByLevel(int level) {
        switch (level) {
            case 1:
                return "男爵";
            case 2:
                return "子爵";
            case 3:
                return "伯爵";
            case 4:
                return "侯爵";
            case 5:
                return "公爵";
            case 6:
                return "国王";
            case 7:
                return "帝皇";
            default:
                return "未知";
        }
    }

    /**
     * 通过贵族卡开通贵族
     *
     * @param userId
     * @param bagId
     * @param channelId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean newNobleByBagId(Long userId, String bagId, Long channelId) {
        BagItem<PropsVO> bagItem = bagService.getProps(bagId, userId);
        if (bagItem == null) {
            return false;
        }

        Integer nobleLevel = bagItem.getItem().getPropsExtInt();
        if (nobleLevel == null) {
            return false;
        }

        Long curTime = System.currentTimeMillis();

        NoblePriceConfigDO noblePriceConfig = noblePriceConfigDAO.selectByLevel(nobleLevel);
        if (noblePriceConfig == null) {
            return false;
        }

        // 写入记录
        Long hisId = identityService.getId();
        Long duration = bagItem.getItem().getPropsExtLong();

        UserNobleHisDO userNobleHis = new UserNobleHisDO();
        userNobleHis.setHisId(hisId);
        userNobleHis.setUserId(userId);
        userNobleHis.setNobleLevel(nobleLevel);
        userNobleHis.setHisAmount(0L);
        userNobleHis.setHisAmountType(noblePriceConfig.getNoblePriceType());
        userNobleHis.setHisType(NoblePayTypeEnum.EXCHANGE.getValue());
        userNobleHis.setHisDuration(duration);
        userNobleHis.setHisCreateTime(curTime);
        userNobleHis.setHisTimes(1);
        userNobleHis.setHisAutoRenewal(BoolIntVal.False);
        userNobleHis.setChannelId(channelId);
        userNobleHisDAO.insert(userNobleHis);

        // 更新贵族
        Long expireTime = duration > 0 ? System.currentTimeMillis() + duration : duration;
        UserNobleDO userNoble = new UserNobleDO();
        userNoble.setUserId(userId);
        userNoble.setNobleLevel(nobleLevel);
        userNoble.setAutoRenewal(BoolIntVal.True);
        userNoble.setExpireTime(expireTime);
        userNoble.setStatus(CommonStatus.Normal);
        userNoble.setUpdateTime(curTime);
        userNoble.setChannelId(channelId);
        userNobleDAO.insertOrUpdate(userNoble);

        // 更新背包
        UserBagDO userBagDO = userBagDAO.selectById(bagId);
        int updated = userBagDAO.updateQuantity(bagId, userBagDO.getQuantity() - 1, userBagDO.getVersion(), curTime);
        if (updated == 0) {
            throw new RuntimeException("更新贵族卡数量失败");
        }

        // 写入变更记录
        UserBagHisDO userBagHisDO = new UserBagHisDO();
        userBagHisDO.setHisId(identityService.getId());
        userBagHisDO.setBagId(bagId);
        userBagHisDO.setItemId(userBagDO.getItemId());
        userBagHisDO.setUserId(userBagDO.getUserId());
        userBagHisDO.setItemType(userBagDO.getItemType());
        userBagHisDO.setQuantity(1);
        userBagHisDO.setCreateTime(curTime);
        userBagHisDO.setHisRemarks(String.format("兑换%s贵族", NobleLevelEnum.toEnum(nobleLevel)));
        userBagHisDO.setOrderId(hisId);
        userBagHisDAO.insert(userBagHisDO);

        // 丢出消息
        QueueNobleNewMsg nobleNewMsg = new QueueNobleNewMsg();
        nobleNewMsg.setUserId(userId);
        nobleNewMsg.setNobleLevel(nobleLevel);
        nobleNewMsg.setChannelId(channelId);
        nobleNewMsg.setIsFromProps(true);
        nobleNewMsg.setDuration(duration);
        nobleNewMsg.setPropsId(bagItem.getItem().getPropsId());
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.NOBLE_NEW, nobleNewMsg.serialize());

        if (expireTime > 0) {
            long deliveryStartTime = duration > TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : expireTime;
            // 丢出贵族过期消息
            QueueNobleExpireMsg nobleExpireMsg = new QueueNobleExpireMsg();
            nobleExpireMsg.setUserId(userId);
            nobleExpireMsg.setHisId(hisId);
            nobleExpireMsg.setNobleLevel(nobleLevel);
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NOBLE_EXPIRE, nobleExpireMsg.serialize(), deliveryStartTime);
        }

        return true;
    }

    /**
     * 续费贵族
     *
     * @param userId
     * @param isAutoRenewal
     * @param times
     * @param channelId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long renewNoble(Long userId, boolean isAutoRenewal, int times, Long channelId) {
        UserNobleDO userNoble = userNobleDAO.selectById(userId, null);
        if (userNoble == null) {
            return null;
        }

        NoblePriceConfigDO noblePriceConfig = getPriceConfigByLevel(userNoble.getNobleLevel());
        if (noblePriceConfig == null) {
            return null;
        }

        Long amount = isAutoRenewal ? noblePriceConfig.getAutoRenewalPrice() : noblePriceConfig.getRenewalPrice();
        amount = times * amount;

        Long hisId = identityService.getId();
        Long balance = walletService.reduceCoin(userId, noblePriceConfig.getNoblePriceType(), amount, String.format("续费贵族%s%d次", NobleLevelEnum.toEnum(userNoble.getNobleLevel()).getDesc(), times), hisId);
        if (balance < 0) {
            return balance;
        }

        // 写入记录
        Long curTime = System.currentTimeMillis();
        Long duration = noblePriceConfig.getDuration();
        UserNobleHisDO userNobleHis = new UserNobleHisDO();
        userNobleHis.setHisId(hisId);
        userNobleHis.setUserId(userId);
        userNobleHis.setNobleLevel(userNoble.getNobleLevel());
        userNobleHis.setHisAmount(amount);
        userNobleHis.setHisAmountType(noblePriceConfig.getNoblePriceType());
        userNobleHis.setHisType(NoblePayTypeEnum.RENEW.getValue());
        userNobleHis.setHisDuration(duration);
        userNobleHis.setHisCreateTime(curTime);
        userNobleHis.setHisTimes(times);
        userNobleHis.setHisAutoRenewal(isAutoRenewal ? BoolIntVal.True : BoolIntVal.False);
        userNobleHisDAO.insert(userNobleHis);

        // 更新贵族
        Long expireTime = userNoble.getExpireTime() + duration * times;
        userNoble.setExpireTime(expireTime);
        userNoble.setStatus(CommonStatus.Normal);
        userNoble.setUpdateTime(curTime);
        userNobleDAO.insertOrUpdate(userNoble);

        // 丢出消息
        QueueNobleRenewalMsg nobleRenewalMsg = new QueueNobleRenewalMsg();
        nobleRenewalMsg.setUserId(userId);
        nobleRenewalMsg.setNobleLevel(userNoble.getNobleLevel());
        nobleRenewalMsg.setTimes(times);
        nobleRenewalMsg.setChannelId(channelId);
        nobleRenewalMsg.setDuration(duration);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.NOBLE_RENEW, nobleRenewalMsg.serialize());

        if (expireTime > 0) {
            long deliveryStartTime = expireTime - curTime > TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : expireTime;

            // 丢出贵族过期消息
            QueueNobleExpireMsg nobleExpireMsg = new QueueNobleExpireMsg();
            nobleExpireMsg.setUserId(userId);
            nobleExpireMsg.setHisId(hisId);
            nobleExpireMsg.setNobleLevel(userNoble.getNobleLevel());
            rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NOBLE_EXPIRE, nobleExpireMsg.serialize(), deliveryStartTime);
        }

        return balance;
    }

    /**
     * 升级贵族
     *
     * @param userId
     * @param nobleLevel
     * @param channelId
     * @return
     */
    @Override
    public Long upgradeNoble(Long userId, int nobleLevel, Long channelId) {

        UserNobleDO userNoble = userNobleDAO.selectById(userId, System.currentTimeMillis());
        if (userNoble == null) {
            return null;
        }
        int oldNobleLevel = userNoble.getNobleLevel();

        NobleUpgradeAmount nobleUpgradeAmount = getUpgradeAmount(userId, nobleLevel);
        if (nobleUpgradeAmount == null) {
            return null;
        }

        Long hisId = identityService.getId();
        Long balance = walletService.reduceCoin(userId, nobleUpgradeAmount.getPriceType(), nobleUpgradeAmount.getAmount(), String.format("升级贵族%s", NobleLevelEnum.toEnum(nobleLevel).getDesc()), hisId);
        if (balance < 0) {
            return balance;
        }

        UserNobleDO userNobleDO = userNobleDAO.selectById(userId, System.currentTimeMillis());

        // 写入记录
        Long curTime = System.currentTimeMillis();
        UserNobleHisDO userNobleHis = new UserNobleHisDO();
        userNobleHis.setHisId(hisId);
        userNobleHis.setUserId(userId);
        userNobleHis.setNobleLevel(nobleLevel);
        userNobleHis.setHisAmount(nobleUpgradeAmount.getAmount());
        userNobleHis.setHisAmountType(nobleUpgradeAmount.getPriceType());
        userNobleHis.setHisType(NoblePayTypeEnum.UPGRADE.getValue());
        if (userNobleDO != null) {
            userNobleHis.setHisDuration(userNobleDO.getExpireTime() - System.currentTimeMillis());
        } else {
            userNobleHis.setHisDuration(0L);
        }
        userNobleHis.setHisCreateTime(curTime);
        userNobleHis.setHisTimes(1);
        userNobleHis.setHisAutoRenewal(BoolIntVal.False);
        userNobleHisDAO.insert(userNobleHis);

        // 更新贵族
        userNoble.setNobleLevel(nobleLevel);
        userNoble.setUpdateTime(curTime);
        userNobleDAO.insertOrUpdate(userNoble);

        // 丢出消息
        QueueNobleUpgradeMsg nobleUpgradeMsg = new QueueNobleUpgradeMsg();
        nobleUpgradeMsg.setUserId(userId);
        nobleUpgradeMsg.setOldNobleLevel(oldNobleLevel);
        nobleUpgradeMsg.setNewNobleLevel(nobleLevel);
        nobleUpgradeMsg.setChannelId(channelId);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.NOBLE_UPGRADE, nobleUpgradeMsg.serialize());

        return balance;
    }

    /**
     * 获取升级差价
     *
     * @param userId
     * @param nobleLevel
     * @return
     */
    @Override
    public NobleUpgradeAmount getUpgradeAmount(Long userId, int nobleLevel) {

        UserNobleDO userNoble = userNobleDAO.selectById(userId, null);
        if (userNoble == null || nobleLevel <= userNoble.getNobleLevel()) {
            return null;
        }

        NoblePriceConfigDO noblePriceConfig = getPriceConfigByLevel(nobleLevel);
        if (noblePriceConfig == null) {
            return null;
        }

        NoblePriceConfigDO curNoblePriceConfig = getPriceConfigByLevel(userNoble.getNobleLevel());
        if (curNoblePriceConfig == null) {
            return null;
        }

        if (!noblePriceConfig.getNoblePriceType().equals(curNoblePriceConfig.getNoblePriceType())) {
            return null;
        }

        // 计算差价
        long dateDiff = DateUtil.between(new Date(userNoble.getExpireTime()), new Date(), DateUnit.DAY);
        if (dateDiff < 1) {
            return null;
        }
        // 每日价格
        long dayPrice = BigDecimal.valueOf(noblePriceConfig.getNoblePrice()).divide(BigDecimal.valueOf(noblePriceConfig.getDuration()).divide(BigDecimal.valueOf(TimeConstants.DAY_MILLS)), 0, RoundingMode.UP).longValue();
        long curDayPrice = BigDecimal.valueOf(curNoblePriceConfig.getNoblePrice()).divide(BigDecimal.valueOf(curNoblePriceConfig.getDuration()).divide(BigDecimal.valueOf(TimeConstants.DAY_MILLS)), 0, RoundingMode.UP).longValue();
        log.info("dateDiff {} dayPrice {} curDayPrice {}", dateDiff, dayPrice, curDayPrice);

        NobleUpgradeAmount nobleUpgradeAmount = new NobleUpgradeAmount();
        nobleUpgradeAmount.setAmount(dateDiff * (dayPrice - curDayPrice));
        nobleUpgradeAmount.setDateDiff(dateDiff);
        nobleUpgradeAmount.setPriceType(curNoblePriceConfig.getNoblePriceType());
        return nobleUpgradeAmount;
    }

    /**
     * 设置自动续费
     *
     * @param userId
     * @param autoRenewal
     * @return
     */
    @Override
    public boolean setAutoRenewal(Long userId, int autoRenewal) {
        try {
            userNobleDAO.updateAutoRenewal(userId, autoRenewal);
            return true;
        } catch (Exception e) {
            log.error("自动续费设置失败", e);
        }
        return false;
    }
}
