package com.starbuds.server.service.user.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.rtc.AuthRtcConfig;
import com.starbuds.server.common.pojo.rtc.RtcLiveUserProfile;
import com.starbuds.server.common.pojo.rtc.RtcRoomCategory;
import com.starbuds.server.common.pojo.rtc.RtcRoomType;
import com.starbuds.server.common.pojo.user.LiveUserProfile;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.RtcUserProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * RTC 用户服务实现
 *
 * @author Lay
 * @date 2020/7/10
 */
@Service(version = "1.0.0")
@Slf4j
public class RtcUserServiceImpl implements RtcUserService {

    @Resource
    RtcUserProvider rtcUserProvider;

    @Resource
    RtcRoomVisitHisDAO rtcRoomVisitHisDAO;

    @Resource
    UserRtcConfigDAO userRtcConfigDAO;

    @Resource
    ChannelRtcConfigDAO channelRtcConfigDAO;

    @Resource
    FamilyRtcConfigDAO familyRtcConfigDAO;

    @Resource
    ChannelDAO channelDAO;

    @Resource
    RtcRoomCategoryDAO rtcRoomCategoryDAO;

    @Resource
    private UserNobleDAO userNobleDAO;

    /**
     * 获取用户在直播间的资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    @Override
    public LiveUserProfile getUserProfile(Long userId, Long roomId) {
        return rtcUserProvider.getRtcUserProfile(userId, roomId);
    }

    /**
     * 缓存用户直播间资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    @Override
    public LiveUserProfile cacheUserProfile(Long userId, Long roomId) {
        return rtcUserProvider.cacheRtcUserProfile(userId, roomId);
    }

    /**
     * 移除用户在直播间资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    @Override
    public boolean removeUserProfile(Long userId, Long roomId) {
        return rtcUserProvider.removeRtcUserProfile(userId, roomId);
    }

    /**
     * 刷新房间缓存
     *
     * @param userId
     * @return
     */
    @Override
    public boolean refreshRoomProfile(Long userId) {
        return rtcUserProvider.refreshRtcUserProfile(userId);
    }

    /**
     * 增加访问记录
     *
     * @param roomId
     * @param userId
     * @param timestamp
     * @return
     */
    @Override
    public boolean addVisitHis(Long roomId, Long userId, Long timestamp) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Integer hisDate = Convert.toInt(sdf.format(new Date(timestamp)));
        String hisId = SecureUtil.md5(String.format("%d:%d:%d", userId, roomId, hisDate));

        RtcRoomVisitHisDO roomVisitHis = new RtcRoomVisitHisDO();
        roomVisitHis.setHisId(hisId);
        roomVisitHis.setHisDate(hisDate);
        roomVisitHis.setRoomId(roomId);
        roomVisitHis.setUserId(userId);
        roomVisitHis.setHisCreateTime(timestamp);
        roomVisitHis.setHisUpdateTime(timestamp);
        roomVisitHis.setStatus(CommonStatus.Normal);
        roomVisitHis.setHisTimes(1);
        rtcRoomVisitHisDAO.insertOrUpdate(roomVisitHis);

        return true;
    }

    @Override
    public List<RtcLiveUserProfile> getUserProfiles(Long roomId) {
        List<LiveUserProfile> rtcUserList = rtcUserProvider.getRtcUserProfiles(roomId);
        if (isDev()) {
            log.info("rtc-User-List {}", JSON.toJSONString(rtcUserList, true));
        }
        if (CollectionUtils.isEmpty(rtcUserList)) {
            return Collections.emptyList();
        }

        Set<Long> userIds = rtcUserList.stream()
                .filter(Objects::nonNull)
                .map(LiveUserProfile::getUserId)
                .filter(Objects::nonNull)
                .collect(Collectors.toSet());
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }

        Map<Long, UserNobleDO> nobleMap = userNobleDAO.selectByUserIds(userIds, System.currentTimeMillis());
        List<RtcLiveUserProfile> list = new ArrayList<>();
        for (LiveUserProfile profile : rtcUserList) {
            if (profile == null) {
                continue;
            }
            Long userId = profile.getUserId();
            if (userId == null) {
                continue;
            }
            RtcLiveUserProfile rtcProfile = new RtcLiveUserProfile();
            BeanUtils.copyProperties(profile, rtcProfile);
            rtcProfile.setIsNoble(false);
            UserNobleDO noble = nobleMap.get(userId);
            if (noble != null) {
                rtcProfile.setIsNoble(true);
                rtcProfile.setNobleLevel(noble.getNobleLevel());
            }
            list.add(rtcProfile);
        }
        return list;
    }

    @Override
    public List<Long> getOnlineUserIds(Long roomId) {
        return rtcUserProvider.getRtcUserIds(roomId);
    }

    @Override
    public AuthRtcConfig getAuthRtcConfig(Long userId, Long channelId) {
        return getAuthRtcConfig(userId, channelId, RtcBusinessTypeEnum.RTC_NORMAL.getValue());
    }

    @Override
    public AuthRtcConfig getAuthRtcConfig(Long userId, Long channelId, Integer businessType) {
        AuthRtcConfig rtcConfig = new AuthRtcConfig();
        UserRtcConfigDO userRtcConfigDO = userRtcConfigDAO.selectRtcConfigById(userId, businessType);
        if (userRtcConfigDO == null) {
            Long configId = 0L;
            if (businessType.equals(RtcBusinessTypeEnum.RTC_NORMAL.getValue())) {
                configId = channelDAO.selectRtcConfigId(channelId);
                // 如果没有，默认取渠道0的配置
                if (configId == null) {
                    configId = channelDAO.selectRtcConfigId(0L);
                }
            } else {
                configId = channelDAO.selectSingleRtcConfigId(channelId);
                // 如果没有，默认取渠道0的配置
                if (configId == null) {
                    configId = channelDAO.selectSingleRtcConfigId(0L);
                }
            }

            ChannelRtcConfigDO channelConfigDO = channelRtcConfigDAO.selectRtcConfigById(configId);
            if (channelConfigDO == null) {
                return null;
            }
            rtcConfig.setBusinessType(channelConfigDO.getBusinessType());
            rtcConfig.setRoomType(channelConfigDO.getRoomType());
            rtcConfig.setRoomCategory(channelConfigDO.getRoomCategory());
            rtcConfig.setRoomNum(Optional.ofNullable(channelConfigDO.getRoomNum()).orElse(0));
        } else {
            rtcConfig.setBusinessType(userRtcConfigDO.getBusinessType());
            rtcConfig.setRoomType(userRtcConfigDO.getRoomType());
            rtcConfig.setRoomCategory(userRtcConfigDO.getRoomCategory());
            rtcConfig.setRoomNum(Optional.ofNullable(userRtcConfigDO.getRoomNum()).orElse(0));
        }
        // 分类
        String roomCategory = rtcConfig.getRoomCategory();
        if (StringUtils.isNotEmpty(roomCategory)) {
            List<RtcRoomCategory> rtcRoomCategories = new ArrayList<>();
            long[] categoryIds = Arrays.stream(roomCategory.split(",")).mapToLong(Long::valueOf).toArray();
            List<RtcRoomCategoryDO> categoryList = rtcRoomCategoryDAO.findByIds(categoryIds, businessType);
            if (!CollectionUtils.isEmpty(categoryList)) {
                for (RtcRoomCategoryDO item : categoryList) {
                    rtcRoomCategories.add(new RtcRoomCategory(item.getCategoryId(), item.getCategoryName(), item.getLayout(), item.getMakeFriend(), item.getBackgroundColor(), item.getBackgroundImage(), item.getBattleEnable(), item.getBusinessType()));
                }
            }
            rtcConfig.setRtcRoomCategories(rtcRoomCategories);
        }
        // 类型
        String roomType = rtcConfig.getRoomType();
        if (StringUtils.isNotEmpty(roomType)) {
            List<RtcRoomType> rtcRoomTypes = new ArrayList<>();
            List<Integer> roomTypeList = Arrays.stream(roomType.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toList());
            for (Integer item : roomTypeList) {
                rtcRoomTypes.add(new RtcRoomType(businessType, item, RoomTypeEnum.toEnum(item).getDesc()));
            }
            rtcConfig.setRtcRoomTypes(rtcRoomTypes);
        }
        return rtcConfig;
    }

    /**
     * 获取家族语聊配置
     *
     * @param familyId
     * @return
     */
    @Override
    public AuthRtcConfig getAuthRtcConfig(Long familyId) {
        FamilyRtcConfigDO rtcConfigDO = familyRtcConfigDAO.selectRtcConfigById(familyId);
        if (rtcConfigDO == null) {
            return null;
        }
        AuthRtcConfig rtcConfig = new AuthRtcConfig();
        rtcConfig.setRoomType(rtcConfigDO.getRoomType());
        rtcConfig.setRoomCategory(rtcConfigDO.getRoomCategory());
        rtcConfig.setRoomNum(Optional.ofNullable(rtcConfigDO.getRoomNum()).orElse(0));
        // 分类
        String roomCategory = rtcConfig.getRoomCategory();
        if (StringUtils.isNotEmpty(roomCategory)) {
            List<RtcRoomCategory> rtcRoomCategories = new ArrayList<>();
            long[] categoryIds = Arrays.stream(roomCategory.split(",")).mapToLong(Long::valueOf).toArray();
            List<RtcRoomCategoryDO> categoryList = rtcRoomCategoryDAO.findByIds(categoryIds, RtcBusinessTypeEnum.RTC_NORMAL.getValue());
            if (!CollectionUtils.isEmpty(categoryList)) {
                for (RtcRoomCategoryDO item : categoryList) {
                    rtcRoomCategories.add(new RtcRoomCategory(item.getCategoryId(), item.getCategoryName(), item.getLayout(), item.getMakeFriend(), item.getBackgroundColor(), item.getBackgroundImage(), item.getBattleEnable()));
                }
            }
            rtcConfig.setRtcRoomCategories(rtcRoomCategories);
        }
        // 类型
        String roomType = rtcConfig.getRoomType();
        if (StringUtils.isNotEmpty(roomType)) {
            List<RtcRoomType> rtcRoomTypes = new ArrayList<>();
            List<Integer> roomTypeList = Arrays.stream(roomType.split(",")).map(e -> Convert.toInt(e)).collect(Collectors.toList());
            for (Integer item : roomTypeList) {
                rtcRoomTypes.add(new RtcRoomType(RtcBusinessTypeEnum.RTC_NORMAL.getValue(), item, RoomTypeEnum.toEnum(item).getDesc()));
            }
            rtcConfig.setRtcRoomTypes(rtcRoomTypes);
        }
        return rtcConfig;
    }

    @Override
    public int getRtcUserJoinDayBefore(Long userId, Long roomId, Long timestamp) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(timestamp);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        long beginTime = calendar.getTimeInMillis();
        return rtcRoomVisitHisDAO.countByUserIdWithTime(userId, roomId, beginTime, timestamp);
    }

    @Override
    public int getRtcRoomJoinUsersBefore(Long roomId, Long beginTime) {
        return rtcRoomVisitHisDAO.countByRoomIdWithTime(roomId, beginTime);
    }

    /**
     * 获取房间进入用户数(去重)
     *
     * @param beginTime
     * @param endTime
     * @param businessType
     * @return
     */
    @Override
    public int getRtcRoomEnterTimes(Long beginTime, Long endTime, Integer businessType) {
        return rtcRoomVisitHisDAO.getRtcRoomEnterTimes(beginTime, endTime, businessType);
    }

    /**
     * 判断用户是否在房间
     *
     * @param roomId
     * @param userId
     * @return
     */
    @Override
    public boolean isUserInRoom(Long roomId, Long userId) {
        return rtcUserProvider.isInRoom(roomId, userId);
    }

    /**
     * 获取用户当前所在房间
     *
     * @param userId
     * @return
     */
    @Override
    public Long getUserStayRoomId(Long userId) {
        return rtcUserProvider.getUserRoomId(userId);
    }

    /**
     * 获得用户房间Map
     *
     * @param userIds 用户id集合
     * @return userId:roomId
     */
    @Override
    public Map<Long, Long> getUserRoomIdMap(Collection<Long> userIds) {
        return rtcUserProvider.getUserRoomIdMap(userIds);
    }
}
