package com.starbuds.server.service.user.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.pojo.activity.MedalPersonRankVO;
import com.starbuds.server.common.pojo.db.DataLevelScoreDO;
import com.starbuds.server.common.pojo.db.UserScoreDO;
import com.starbuds.server.common.pojo.user.ScoreLevelInfo;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.user.dao.UserScoreDAO;
import com.starbuds.server.service.user.provider.ScoreProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/25 14:19
 * @Description 积分服务
 */
@Slf4j
@Service(version = "1.0.0")
public class ScoreServiceImpl implements ScoreService {

    @Reference(version = "1.0.0")
    DataService dataService;


    @Resource
    private ScoreProvider scoreProvider;

    @Resource
    UserScoreDAO userScoreDAO;

    /**
     * 创建积分账户
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createScoreAccount(Long userId) {
        try {
            long currentTime = System.currentTimeMillis();
            for (ScoreTypeEnum item : ScoreTypeEnum.values()) {
                UserScoreDO scoreDO = new UserScoreDO();
                scoreDO.setScoreId(SecureUtil.md5(String.format("%d:%d", userId, item.getValue())));
                scoreDO.setUserId(userId);
                scoreDO.setScoreType(item.getValue());
                scoreDO.setScoreValue(0L);
                scoreDO.setVersion(1L);
                scoreDO.setCreateTime(currentTime);
                scoreDO.setUpdateTime(currentTime);
                userScoreDAO.insert(scoreDO);
            }
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("积分账户创建失败", e);
        }
    }

    /**
     * 增加积分
     *
     * @param userId
     * @param scoreType
     * @param scoreValue
     * @param remarks
     * @param businessId
     * @param anchorUserId
     * @return
     */
    @Override
    public boolean addScore(Long userId, Integer scoreType, Long scoreValue, String remarks, Long businessId, Long anchorUserId) {
        return scoreProvider.addScore(userId, ScoreTypeEnum.toEnum(scoreType), scoreValue, remarks, businessId, anchorUserId);
    }

    /**
     * 减少积分
     *
     * @param userId
     * @param scoreType
     * @param scoreValue
     * @param remarks
     * @param businessId
     * @return
     */
    @Override
    public Long reduceScore(Long userId, Integer scoreType, Long scoreValue, String remarks, Long businessId) {
        return scoreProvider.reduceScore(userId, ScoreTypeEnum.toEnum(scoreType), scoreValue, remarks, businessId);
    }

    /**
     * 获取对应的积分值
     *
     * @param userId
     * @param scoreTypeEnum
     * @return
     */
    @Override
    public Long getScoreValue(Long userId, ScoreTypeEnum scoreTypeEnum) {
        return scoreProvider.getScoreValue(userId, scoreTypeEnum);
    }

    /**
     * 获取对应的积分值
     *
     * @param userIds
     * @param scoreTypeEnum
     * @return
     */
    @Override
    public Long getScoreValues(List<Long> userIds, ScoreTypeEnum scoreTypeEnum) {
        return scoreProvider.getScoreValues(userIds, scoreTypeEnum);
    }

    /**
     * 获取用户等级积分详情
     *
     * @param userId
     * @param scoreType
     * @return
     */
    @Override
    public ScoreLevelInfo getScoreLevelInfo(Long userId, ScoreTypeEnum scoreType) {
        ScoreLevelInfo scoreLevelInfo = new ScoreLevelInfo();
        scoreLevelInfo.setScoreType(scoreType.getValue());

        // 查询用户积分等级
        Long scoreValue = scoreProvider.getScoreValue(userId, scoreType);
        scoreLevelInfo.setScoreValue(scoreValue);
        int curLevel = scoreProvider.getScoreLevel(scoreValue, scoreType);
        scoreLevelInfo.setLevel(scoreProvider.getScoreLevel(scoreValue, scoreType));
        scoreLevelInfo.setLevelScoreValue(dataService.getLevelScore(curLevel, scoreType));

        // 查询下一等级
        DataLevelScoreDO nextLevel = dataService.getNextLevelScore(curLevel, scoreType);
        if (nextLevel == null) {
            // 已满级
            scoreLevelInfo.setNextLevel(-1);
            scoreLevelInfo.setNextScoreValue(0L);
        } else {
            scoreLevelInfo.setNextLevel(nextLevel.getScoreLevel());
            scoreLevelInfo.setNextScoreValue(nextLevel.getScoreValue());
        }
        return scoreLevelInfo;
    }

    /**
     * 获取活动勋章等级详情
     *
     * @param userId
     * @return
     */
    @Override
    public List<String> getScoreLevelIds(Long userId) {
        // 获取所有活动列表
        List<DataLevelScoreDO> activitiesMedalList = dataService.getActivitiesMedalList();
        if (CollectionUtil.isEmpty(activitiesMedalList)) {
            return Collections.EMPTY_LIST;
        }
        List<DataLevelScoreDO> activitiesMedalSortedList = activitiesMedalList.stream().sorted(Comparator.comparing(DataLevelScoreDO::getSort)).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(activitiesMedalSortedList)) {
            return Collections.EMPTY_LIST;
        }
        Set<Integer> scoreTypes = activitiesMedalList.stream().map(DataLevelScoreDO::getScoreType).collect(Collectors.toSet());

        List<String> ids = new ArrayList<>();

        scoreTypes.forEach(val -> {
            ScoreTypeEnum scoreType = ScoreTypeEnum.toEnum(val);
            if (scoreType == null) {
                return;
            }
            Long scoreValue = scoreProvider.getScoreValue(userId, scoreType);
            int curLevel = scoreProvider.getScoreLevelDefault(BigDecimal.valueOf(scoreValue).divide(BigDecimal.valueOf(100)).longValue(), scoreType);

            Optional<DataLevelScoreDO> scoreDO = activitiesMedalSortedList.stream().filter(o -> o.getScoreType().equals(val)).filter(o -> o.getScoreLevel().equals(curLevel)).findFirst();
            if (!scoreDO.isPresent()) {
                return;
            }

            DataLevelScoreDO score = scoreDO.get();
            long now = System.currentTimeMillis();
            if (now <= score.getEndTime() || score.getEndTime() == BoolIntVal.Ignore) {
                // 送礼成就判断是不是教皇
                if (ScoreTypeEnum.GIFT_SEND_21.equals(scoreType)) {

                    // 是否是倒数第二级
                    DataLevelScoreDO lastSecondLevel = activitiesMedalSortedList.get(activitiesMedalList.size() - 2);
                    DataLevelScoreDO lastLevel = activitiesMedalSortedList.get(activitiesMedalList.size() - 1);
                    if (score.getScoreLevel().equals(lastSecondLevel.getScoreLevel())) {

                        // 获取排名榜
                        if (scoreProvider.isTopRankAndScoreEnough(userId)) {
                            ids.add(lastLevel.getScoreId());
                            return;
                        }
                    }
                    ids.add(score.getScoreId());
                } else {
                    ids.add(score.getScoreId());
                }
            }
        });

        return ids;
    }


    /**
     * 获取用户积分等级排名
     *
     * @param scoreType
     * @return
     */
    @Override
    public List<MedalPersonRankVO> getActivitiesMedalPersonRankList(ScoreTypeEnum scoreType) {

        List<MedalPersonRankVO> resultSorted = scoreProvider.getActivitiesMedalPersonRankResourceList(scoreType);

        // 获取所有活动列表
        List<DataLevelScoreDO> activitiesMedalList = dataService.getActivitiesMedalList();
        if (CollectionUtil.isEmpty(activitiesMedalList)) {
            return Collections.EMPTY_LIST;
        }
        List<DataLevelScoreDO> activitiesMedalSortedList = activitiesMedalList.stream().filter(o -> o.getScoreType().equals(scoreType.getValue())).sorted(Comparator.comparing(DataLevelScoreDO::getSort)).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(activitiesMedalSortedList)) {
            return Collections.EMPTY_LIST;
        }
        // 排名排序并且判断是否教皇,积分此时已经是转换后的积分
        for (int i = 0; i < resultSorted.size(); i++) {
            MedalPersonRankVO vo = resultSorted.get(i);
            vo.setRankSort(i + 1);
            DataLevelScoreDO currentLevelScore = getCurrentLevelFromList(activitiesMedalSortedList, vo.getScoreValue(), scoreType);

            // 如果是娱乐勋章排名, 就要区分教皇
            if (ScoreTypeEnum.GIFT_SEND_21.equals(scoreType)) {

                if (i <= 2) {
                    DataLevelScoreDO lastLevel = activitiesMedalSortedList.get(activitiesMedalSortedList.size() - 1);
                    boolean isTopRankAndScoreEnough = scoreProvider.isTopRankAndScoreEnough(resultSorted, vo.getUserId(), vo.getScoreValue(), lastLevel);
                    if (isTopRankAndScoreEnough) {
                        vo.setLevelImg(lastLevel.getLevelImg());
                        vo.setScoreId(lastLevel.getScoreId());
                    } else {
                        vo.setLevelImg(currentLevelScore.getLevelImg());
                        vo.setScoreId(currentLevelScore.getScoreId());
                    }
                } else {
                    vo.setLevelImg(currentLevelScore.getLevelImg());
                    vo.setScoreId(currentLevelScore.getScoreId());
                }
            } else {
                // 偶像勋章排名不需要区分
                vo.setLevelImg(currentLevelScore.getLevelImg());
                vo.setScoreId(currentLevelScore.getScoreId());
            }

        }

        return resultSorted;
    }


    /**
     * 获取当前等级
     *
     * @param activitiesMedalSortedList
     * @param scoreValue
     * @return
     */
    private DataLevelScoreDO getCurrentLevelFromList(List<DataLevelScoreDO> activitiesMedalSortedList, Long scoreValue, ScoreTypeEnum scoreType) {

        DataLevelScoreDO current = new DataLevelScoreDO();
        List<DataLevelScoreDO> activitiesMedalSortedASCList = activitiesMedalSortedList.stream().sorted(Comparator.comparing(DataLevelScoreDO::getScoreValue)).collect(Collectors.toList());
        for (int i = 0; i < activitiesMedalSortedASCList.size(); i++) {
            DataLevelScoreDO levelScore = activitiesMedalSortedASCList.get(i);
            if (ScoreTypeEnum.GIFT_SEND_21.equals(scoreType)) {
                if (i == activitiesMedalSortedASCList.size() - 1) {
                    break;
                }
            }
            if (scoreValue >= levelScore.getScoreValue()) {
                BeanUtils.copyProperties(levelScore, current);
            }
        }

        return current;
    }


}
