package com.starbuds.server.service.user.impl;

import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayConstants;
import com.alipay.api.CertAlipayRequest;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.cloudauth.model.v20190307.*;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.user.ImgTypeEnum;
import com.starbuds.server.common.pojo.data.OauthConfig;
import com.starbuds.server.common.pojo.db.UserAlipayDO;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.data.OauthConfigService;
import com.starbuds.server.common.service.user.ThirdAuthService;
import com.starbuds.server.service.user.dao.UserAlipayDAO;
import com.starbuds.server.service.user.provider.IOneKeyLoginProvider;
import com.starbuds.server.service.user.provider.OauthConfigProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 第三方授权服务实现
 *
 * @author Lay
 * @date 2019/12/5
 */
@Slf4j
@Service(version = "1.0.0")
public class ThirdAuthServiceImpl implements ThirdAuthService {

    @Resource
    IOneKeyLoginProvider oneKeyLoginProvider;

    @Resource
    OauthConfigProvider oauthConfigProvider;

    @Reference(version = "1.0.0")
    OauthConfigService oauthConfigService;

    @Resource
    UserAlipayDAO userAlipayDAO;

    @Value("${alipay.appId}")
    String alipayAppId;

    @Value("${alipay.privateKey}")
    String alipayPrivateKey;

    @Value("${alipay.pid}")
    String alipayPid;

    @Override
    public String getOneKeyPhoneByCert(Long channelId, String cert, Integer clientType) {
        log.info("入参,channelId:{},cert:{},clientType:{}", channelId, cert, clientType);
        return oneKeyLoginProvider.getMobile(channelId, cert, clientType);
    }

    /**
     * 获取微信 access_token
     *
     * @param channelId
     * @param code
     * @return
     */
    @Override
    public WxAccessTokenResp getWxAppAccessToken(Long channelId, String code) {
        log.info("入参,channelId:{},code:{}", channelId, code);
        ChannelAppConfig appConfig = oauthConfigProvider.getWxAppConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置微信授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置微信授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("appid", oauthConfig.getAppId());
            requestMap.put("secret", oauthConfig.getAppSecret());
            requestMap.put("code", code);
            requestMap.put("grant_type", "authorization_code");
            String response = HttpUtil.get("https://api.weixin.qq.com/sns/oauth2/access_token", requestMap);
            log.info("微信 accessToken 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("微信服务无响应");
                return null;
            }
            JSONObject result = JSON.parseObject(response);
            if (result != null) {
                int errCode = Optional.ofNullable(result.getInteger("errcode")).orElse(0);
                if (errCode != 0) {
                    log.info("获取微信accessToken出错了,{}", JSON.toJSONString(result));
                    return null;
                }
                WxAccessTokenResp wxAccessTokenResp = JSON.parseObject(response, WxAccessTokenResp.class);
                if (wxAccessTokenResp != null) {
                    return wxAccessTokenResp;
                }
            }
        } catch (Exception e) {
            log.info("通过 code 获取 access_token 出错", e);
        }
        return null;
    }

    /**
     * 获取微信用户信息
     *
     * @param channelId
     * @param accessToken
     * @return
     */
    @Override
    public WxAppUserInfoResp getWxAppUserInfo(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig appConfig = oauthConfigProvider.getWxAppConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置微信授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置微信授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            requestMap.put("openid", oauthConfig.getAppId());
            String response = HttpUtil.get("https://api.weixin.qq.com/sns/userinfo", requestMap);
            log.info("微信 userInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("微信服务无响应");
                return null;
            }
            JSONObject result = JSON.parseObject(response);
            if (result != null) {
                int errCode = Optional.ofNullable(result.getInteger("errcode")).orElse(0);
                if (errCode != 0) {
                    log.info("获取微信用户信息出错了,{}", JSON.toJSONString(result));
                    return null;
                }
            }
            WxAppUserInfoResp wxAppUserInfoResp = JSON.parseObject(response, WxAppUserInfoResp.class);
            return wxAppUserInfoResp;
        } catch (Exception e) {
            log.info("获取用户个人信息出错", e);
        }
        return null;
    }

    /**
     * 获取QQ AuthCode
     *
     * @param channelId
     * @return
     */
    @Override
    public String getQqAuthCode(Long channelId) {
        log.info("入参,channelId:{}", channelId);
        ChannelAppConfig appConfig = oauthConfigProvider.getQqConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("response_type", "code");
            requestMap.put("client_id", oauthConfig.getAppId());
            requestMap.put("redirect_uri", oauthConfig.getAuthRedirectUrl());
            requestMap.put("state", CommonStatus.Normal);
            String response = HttpUtil.get("https://graph.qq.com/oauth2.0/authorize", requestMap);
            log.info("QQ authCode 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("QQ服务无响应");
                return null;
            }
            QqAuthCodeResp authCodeResp = JSON.parseObject(parseJSONP(response), QqAuthCodeResp.class);
            if (authCodeResp != null && authCodeResp.getError() == null) {
                return authCodeResp.getCode();
            }
        } catch (Exception e) {
            log.info("获取qq authCode出错", e);
        }
        return null;
    }

    /**
     * 通过AuthCode获取AccessToken
     *
     * @param channelId
     * @param code
     * @return
     */
    @Override
    public QqAccessTokenResp getQqAccessToken(Long channelId, String code) {
        log.info("入参,channelId:{},code:{}", channelId, code);
        ChannelAppConfig appConfig = oauthConfigProvider.getQqConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("grant_type", "authorization_code");
            requestMap.put("client_id", oauthConfig.getAppId());
            requestMap.put("client_secret", oauthConfig.getAppSecret());
            requestMap.put("redirect_uri", oauthConfig.getAccessRedirectUrl());
            requestMap.put("code", code);
            String response = HttpUtil.get("https://graph.qq.com/oauth2.0/token", requestMap);
            log.info("QQ accessToken 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("QQ服务无响应");
                return null;
            }
            QqAccessTokenResp tokenResp = JSON.parseObject(parseJSONP(response), QqAccessTokenResp.class);
            log.info("响应数据，{}", tokenResp);
            if (tokenResp != null && tokenResp.getError() == null) {
                return tokenResp;
            }
        } catch (Exception e) {
            log.info("获取 qq accessToken出错", e);
        }
        return null;
    }

    /**
     * 获取用户QQ openId
     *
     * @param channelId
     * @param accessToken
     * @return
     */
    @Override
    public String getQqOpenId(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig appConfig = oauthConfigProvider.getQqConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            String response = HttpUtil.get("https://graph.qq.com/oauth2.0/me", requestMap);
            log.info("QQ openId 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("QQ服务无响应");
                return null;
            }
            QqOpenIdResp openIdResp = JSON.parseObject(parseJSONP(response), QqOpenIdResp.class);
            if (openIdResp != null && openIdResp.getError() == null) {
                return openIdResp.getOpenId();
            }
        } catch (Exception e) {
            log.info("获取qq用户openId出错", e);
        }
        return null;
    }

    /**
     * 获取qq unionId
     *
     * @param channelId
     * @param accessToken
     * @return
     */
    @Override
    public String getQqUnionId(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig appConfig = oauthConfigProvider.getQqConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            requestMap.put("unionid", 1);
            String response = HttpUtil.get("https://graph.qq.com/oauth2.0/me", requestMap);
            log.info("QQ unionId 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("QQ服务无响应");
                return null;
            }
            QqUnionIdResp unionIdResp = JSON.parseObject(parseJSONP(response), QqUnionIdResp.class);
            if (unionIdResp != null && unionIdResp.getError() == null) {
                return unionIdResp.getUnionId();
            }
        } catch (Exception e) {
            log.info("获取qq unionId出错", e);
        }
        return null;
    }

    /**
     * 根据openId获取QQ用户信息
     *
     * @param channelId
     * @param accessToken
     * @param openId
     * @return
     */
    @Override
    public QqUserInfoResp getQqUserInfo(Long channelId, String accessToken, String openId) {
        log.info("入参,channelId:{},accessToken:{},openId:{}", channelId, accessToken, openId);
        ChannelAppConfig appConfig = oauthConfigProvider.getQqConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置QQ授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            requestMap.put("oauth_consumer_key", oauthConfig.getAppId());
            requestMap.put("openid", openId);
            requestMap.put("format", "json");
            String response = HttpUtil.get("https://graph.qq.com/user/get_user_info", requestMap);
            log.info("QQ userInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("QQ服务无响应");
                return null;
            }
            QqUserInfoResp resp = JSON.parseObject(response, QqUserInfoResp.class);
            if (resp != null && resp.getRet() == 0) {
                return resp;
            }
        } catch (Exception e) {
            log.info("获取qq用户信息出错", e);
        }
        return null;
    }


    /**
     * 获取微博AuthCode
     *
     * @param channelId
     * @return
     */
    @Override
    public String getWbAuthCode(Long channelId) {
        log.info("入参,channelId:{}", channelId);
        ChannelAppConfig appConfig = oauthConfigProvider.getWeiboConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("client_id", oauthConfig.getAppId());
            requestMap.put("redirect_uri", oauthConfig.getAuthRedirectUrl());
            String response = HttpUtil.get("https://api.weibo.com/oauth2/authorize", requestMap);
            log.info("weibo authCode 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("微博服务无响应");
                return null;
            }
            WbAuthCodeResp wbAuthCodeResp = JSON.parseObject(response, WbAuthCodeResp.class);
            if (wbAuthCodeResp != null) {
                return wbAuthCodeResp.getCode();
            }
        } catch (Exception e) {
            log.info("获取微博authCode出错", e);
        }
        return null;
    }

    /**
     * 获取微博AccessToken
     *
     * @param channelId
     * @param code
     * @return
     */
    @Override
    public WbAccessTokenResp getWbAccessToken(Long channelId, String code) {
        log.info("入参,channelId:{},code:{}", channelId, code);
        ChannelAppConfig appConfig = oauthConfigProvider.getWeiboConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("client_id", oauthConfig.getAppId());
            requestMap.put("client_secret", oauthConfig.getAppSecret());
            requestMap.put("grant_type", "authorization_code");
            requestMap.put("code", code);
            requestMap.put("redirect_uri", oauthConfig.getAccessRedirectUrl());
            String response = HttpUtil.get("https://api.weibo.com/oauth2/access_token", requestMap);
            log.info("微博 accessToken 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("微博服务无响应");
                return null;
            }
            WbAccessTokenResp accessTokenResp = JSON.parseObject(response, WbAccessTokenResp.class);
            if (accessTokenResp != null) {
                return accessTokenResp;
            }
        } catch (Exception e) {
            log.info("获取微博accessToken出错", e);
        }
        return null;
    }

    /**
     * 获取微博token信息
     *
     * @param channelId
     * @param accessToken
     * @return
     */
    @Override
    public WbTokenInfoResp getWbTokenInfo(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig wxAppConfig = oauthConfigProvider.getWeiboConfig(channelId);
        if (wxAppConfig == null || wxAppConfig.getConfigId() == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(wxAppConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            String response = HttpUtil.get("https://api.weibo.com/oauth2/get_token_info", requestMap);
            log.info("微博 tokenInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("微博服务无响应");
                return null;
            }
            WbTokenInfoResp accessTokenResp = JSON.parseObject(response, WbTokenInfoResp.class);
            if (accessTokenResp != null) {
                return accessTokenResp;
            }
        } catch (Exception e) {
            log.info("获取微博token信息出错", e);
        }
        return null;
    }

    @Override
    public WbUserInfoResp getWbUserInfo(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig wxAppConfig = oauthConfigProvider.getWeiboConfig(channelId);
        if (wxAppConfig == null || wxAppConfig.getConfigId() == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(wxAppConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置微博授权");
            return null;
        }
        try {
            // 先获取 uid
            String uidUrl = "https://api.weibo.com/2/account/get_uid.json?access_token=" + accessToken;
            String uidRes = HttpUtil.get(uidUrl);
            if (StringUtils.isBlank(uidRes)) {
                log.info("微博服务无响应");
                return null;
            }
            JSONObject uidJson = JSON.parseObject(uidRes);
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            requestMap.put("uid", uidJson.getString("uid"));
            String response = HttpUtil.get("https://api.weibo.com/2/users/show.json", requestMap);
            log.info("微博 userInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("微博服务无响应");
                return null;
            }
            WbUserInfoResp userInfoResp = JSON.parseObject(response, WbUserInfoResp.class);
            if (userInfoResp != null) {
                return userInfoResp;
            }
        } catch (Exception e) {
            log.info("获取微博用户信息出错", e);
        }
        return null;
    }

    /**
     * 获取华为 AccessToken
     *
     * @param channelId
     * @param code
     * @return
     */
    @Override
    public HuaweiAccessTokenResp getHuaweiAccessToken(Long channelId, String code) {
        log.info("入参,channelId:{},code:{}", channelId, code);
        ChannelAppConfig appConfig = oauthConfigProvider.getHuaweiConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置华为授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置华为授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("client_id", oauthConfig.getAppId());
            requestMap.put("client_secret", oauthConfig.getAppSecret());
            requestMap.put("grant_type", "authorization_code");
            requestMap.put("code", code);
            requestMap.put("redirect_uri", oauthConfig.getAccessRedirectUrl());
            String response = HttpUtil.post("https://oauth-login.cloud.huawei.com/oauth2/v3/token", requestMap);
            log.info("华为 accessToken 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("华为服务无响应");
                return null;
            }
            HuaweiAccessTokenResp accessTokenResp = JSON.parseObject(response, HuaweiAccessTokenResp.class);
            if (accessTokenResp != null) {
                return accessTokenResp;
            }
        } catch (Exception e) {
            log.info("获取华为accessToken出错", e);
        }
        return null;
    }

    /**
     * 获取华为 token 信息
     *
     * @param channelId
     * @param accessToken
     * @return
     */
    @Override
    public HuaweiTokenInfoResp getHuaweiTokenInfo(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig appConfig = oauthConfigProvider.getHuaweiConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置华为授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置华为授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            String response = HttpUtil.post("https://api.cloud.huawei.com/rest.php?nsp_fmt=JSON&nsp_svc=huawei.oauth2.user.getTokenInfo", requestMap);
            log.info("华为 tokenInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("华为服务无响应");
                return null;
            }
            HuaweiTokenInfoResp accessTokenResp = JSON.parseObject(response, HuaweiTokenInfoResp.class);
            if (accessTokenResp != null) {
                return accessTokenResp;
            }
        } catch (Exception e) {
            log.info("获取华为token信息出错", e);
        }
        return null;
    }

    /**
     * 获取华为用户信息
     *
     * @param channelId
     * @param idToken
     * @return
     */
    @Override
    public HuaweiInfoResp getHuaweiUserInfo(Long channelId, String idToken) {
        log.info("入参,channelId:{},idToken:{}", channelId, idToken);
        ChannelAppConfig appConfig = oauthConfigProvider.getHuaweiConfig(channelId);
        if (appConfig == null || appConfig.getConfigId() == null) {
            log.info("该渠道未配置华为授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(appConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置华为授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("id_token", idToken);
            String response = HttpUtil.get("https://oauth-login.cloud.huawei.com/oauth2/v3/tokeninfo", requestMap);
            log.info("华为 userInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("华为服务无响应");
                return null;
            }
            HuaweiInfoResp userInfoResp = JSON.parseObject(response, HuaweiInfoResp.class);
            if (userInfoResp != null) {
                return userInfoResp;
            }
        } catch (Exception e) {
            log.info("获取华为用户信息出错", e);
        }
        return null;
    }

    @Override
    public FacebookTokenResp getFacebookAccessToken(Long channelId, String code) {
        log.info("入参,channelId:{},code:{}", channelId, code);
        ChannelAppConfig fbAppConfig = oauthConfigProvider.getFacebookConfig(channelId);
        if (fbAppConfig == null || fbAppConfig.getConfigId() == null) {
            log.info("该渠道未配置 facebook 授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(fbAppConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置 facebook 授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("client_id", oauthConfig.getAppSecret());
            requestMap.put("redirect_uri", oauthConfig.getAuthRedirectUrl());
            requestMap.put("client_secret", oauthConfig.getAppSecret());
            requestMap.put("code", code);
            String response = HttpUtil.get("https://graph.facebook.com/v3.1/oauth/access_token", requestMap);
            log.info("facebook tokenInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("facebook 服务无响应");
                return null;
            }
            FacebookTokenResp accessTokenResp = JSON.parseObject(response, FacebookTokenResp.class);
            if (accessTokenResp != null) {
                return accessTokenResp;
            }
        } catch (Exception e) {
            log.info("获取 facebook token信息出错", e);
        }
        return null;
    }

    @Override
    public FacebookUserInfoResp getFacebookUserInfo(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig fbAppConfig = oauthConfigProvider.getFacebookConfig(channelId);
        if (fbAppConfig == null || fbAppConfig.getConfigId() == null) {
            log.info("该渠道未配置facebook授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(fbAppConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置facebook授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("fields", "id,name,about,address,birthday,favorite_athletes");
            requestMap.put("access_token", accessToken);
            String response = HttpUtil.get("https://graph.facebook.com/v3.1/me", requestMap);
            log.info("facebook userInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info(" facebook 服务无响应");
                return null;
            }
            FacebookUserInfoResp userInfoResp = JSON.parseObject(response, FacebookUserInfoResp.class);
            if (userInfoResp != null) {
                userInfoResp.setClientId(oauthConfig.getAppId());
                return userInfoResp;
            }
        } catch (Exception e) {
            log.info("获取 facebook userInfo 出错", e);
        }
        return null;
    }

    @Override
    public GoogleTokenResp getGoogleAccessToken(Long channelId, String code) {
        log.info("入参,channelId:{},code:{}", channelId, code);
        ChannelAppConfig fbAppConfig = oauthConfigProvider.getFacebookConfig(channelId);
        if (fbAppConfig == null || fbAppConfig.getConfigId() == null) {
            log.info("该渠道未配置 google 授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(fbAppConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置 google 授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("client_id", oauthConfig.getAppSecret());
            requestMap.put("redirect_uri", oauthConfig.getAuthRedirectUrl());
            requestMap.put("client_secret", oauthConfig.getAppSecret());
            requestMap.put("code", code);
            String response = HttpUtil.get("https://www.googleapis.com/oauth2/v4/token", requestMap);
            log.info("google tokenInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info("google 服务无响应");
                return null;
            }
            GoogleTokenResp accessTokenResp = JSON.parseObject(response, GoogleTokenResp.class);
            if (accessTokenResp != null) {
                return accessTokenResp;
            }
        } catch (Exception e) {
            log.info("获取 google token信息出错", e);
        }
        return null;
    }

    @Override
    public GoogleUserInfoResp getGoogleUserInfo(Long channelId, String accessToken) {
        log.info("入参,channelId:{},accessToken:{}", channelId, accessToken);
        ChannelAppConfig goAppConfig = oauthConfigProvider.getGoogleConfig(channelId);
        if (goAppConfig == null || goAppConfig.getConfigId() == null) {
            log.info("该渠道未配置 google 授权");
            return null;
        }
        OauthConfig oauthConfig = oauthConfigService.getAppIdAndSecretConfig(goAppConfig.getConfigId());
        if (oauthConfig == null) {
            log.info("该渠道未配置 google 授权");
            return null;
        }
        try {
            Map<String, Object> requestMap = new HashMap<>();
            requestMap.put("access_token", accessToken);
            String response = HttpUtil.get("https://www.googleapis.com/oauth2/v2/userinfo", requestMap);
            log.info(" google userInfo 返回数据：{}", response);
            if (StringUtils.isBlank(response)) {
                log.info(" google 服务无响应");
                return null;
            }
            GoogleUserInfoResp userInfoResp = JSON.parseObject(response, GoogleUserInfoResp.class);
            if (userInfoResp != null) {
                userInfoResp.setClientId(oauthConfig.getAppId());
                return userInfoResp;
            }
        } catch (Exception e) {
            log.info("获取 google userInfo 出错", e);
        }
        return null;
    }

    /**
     * 是否绑定支付宝账户
     *
     * @param userId
     * @return
     */
    @Override
    public boolean isAlipayBind(Long userId) {
        return userAlipayDAO.countById(userId) > 0;
    }

    /**
     * 获取支付宝账户
     *
     * @param userId
     * @return
     */
    @Override
    public UserAlipayVO getAlipay(Long userId) {
        UserAlipayDO userAlipayDO = userAlipayDAO.selectById(userId);
        if (userAlipayDO != null) {
            UserAlipayVO userAlipayVO = new UserAlipayVO();
            userAlipayVO.setAlipayAccount(userAlipayDO.getAlipayAccount());
            userAlipayVO.setAlipayUserId(userAlipayDO.getAlipayUserId());
            userAlipayVO.setUserId(userId);
            userAlipayVO.setBindTime(userAlipayDO.getUpdateTime());
            return userAlipayVO;
        }
        return null;
    }

    /**
     * 绑定支付宝账户
     *
     * @param userId
     * @param alipayAccount
     * @param code
     * @return
     */
    @Override
    public boolean bindAlipay(Long userId, String alipayAccount, String code) {
        try {
            CertAlipayRequest certAlipayRequest = new CertAlipayRequest();
            certAlipayRequest.setServerUrl("https://openapi.alipay.com/gateway.do");
            certAlipayRequest.setAppId(alipayAppId);
            certAlipayRequest.setPrivateKey(alipayPrivateKey);
            certAlipayRequest.setFormat(AlipayConstants.FORMAT_JSON);
            certAlipayRequest.setCharset(AlipayConstants.CHARSET_UTF8);
            certAlipayRequest.setSignType(AlipayConstants.SIGN_TYPE_RSA2);
            certAlipayRequest.setCertPath("/data/server/cert/app.crt");
            certAlipayRequest.setAlipayPublicCertPath("/data/server/cert/alipay.crt");
            certAlipayRequest.setRootCertPath("/data/server/cert/root.crt");
            AlipayClient alipayClient = new DefaultAlipayClient(certAlipayRequest);
            // 发起请求
            AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
            request.setGrantType("authorization_code");
            request.setCode(code);
            AlipaySystemOauthTokenResponse response = alipayClient.certificateExecute(request);
            if (response.isSuccess()) {
                Long curTime = System.currentTimeMillis();
                UserAlipayDO userAlipayDO = new UserAlipayDO();
                userAlipayDO.setUserId(userId);
                userAlipayDO.setAlipayAccount(alipayAccount);
                userAlipayDO.setAlipayUserId(response.getUserId());
                userAlipayDO.setAccessToken(response.getAccessToken());
                userAlipayDO.setCreateTime(curTime);
                userAlipayDO.setUpdateTime(curTime);
                userAlipayDO.setStatus(CommonStatus.Normal);

                userAlipayDAO.insertOrUpdate(userAlipayDO);
                return true;
            }
        } catch (Exception e) {
            log.error("支付宝账户绑定失败", e);
        }
        return false;
    }

    /**
     * 解绑支付宝账户
     *
     * @param userId
     * @return
     */
    @Override
    public boolean unbindAlipay(Long userId) {
        try {
            userAlipayDAO.updateStatus(userId, CommonStatus.Deleted);
            return true;
        } catch (Exception e) {
            log.error("支付宝账户解绑失败", e);
        }
        return false;
    }

    /**
     * 获取支付宝授权绑定参数
     *
     * @return
     */
    @Override
    public AlipayBindParam getAlipayBindParam(Long userId) {
        String targetId = SecureUtil.md5(String.format("%d%d", userId, System.currentTimeMillis()));
        String infoStr = String.format("apiname=com.alipay.account.auth&app_id=%s&app_name=mc&auth_type=AUTHACCOUNT&biz_type=openservice&method=alipay.open.auth.sdk.code.get&pid=%s&product_id=APP_FAST_LOGIN&scope=kuaijie&sign_type=RSA2&target_id=%s", alipayAppId, alipayPid, targetId);
        try {
            String sign = AlipaySignature.rsa256Sign(infoStr, alipayPrivateKey, AlipayConstants.CHARSET_UTF8);
            infoStr += String.format("&sign=%s", sign);
            AlipayBindParam param = new AlipayBindParam();
            param.setAppId(alipayAppId);
            param.setInfoStr(infoStr);
            return param;
        } catch (Exception e) {
            log.error("支付宝签名失败", e);
        }
        return null;
    }

    /**
     * 获取阿里实人认证token
     *
     * @param channelId
     * @param bizId
     * @param bizType
     * @return
     */
    @Override
    public String getAliRpVerifyToken(Long channelId, String bizId, String bizType) {
        try {
            DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", "AccessKey", "AccessSecret");
            DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", "Cloudauth", "cloudauth.aliyuncs.com");
            IAcsClient client = new DefaultAcsClient(profile);
            DescribeVerifyTokenRequest request = new DescribeVerifyTokenRequest();
            request.setBizId(bizId);
            request.setBizType(bizType);
            DescribeVerifyTokenResponse response = client.getAcsResponse(request);
            if (response != null) {
                return response.getVerifyToken();
            }
        } catch (Exception e) {
            log.info("调用阿里实人认证出错", e);
        }
        return null;
    }

    /**
     * 获取阿里实人认证结果
     *
     * @param channelId
     * @param bizId
     * @param bizType
     * @return
     */
    @Override
    public Boolean getAliRpVerifyResult(Long channelId, String bizId, String bizType) {
        try {
            DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", "AccessKey", "AccessSecret");
            DefaultProfile.addEndpoint("cn-hangzhou", "cn-hangzhou", "Cloudauth", "cloudauth.aliyuncs.com");
            IAcsClient client = new DefaultAcsClient(profile);
            DescribeVerifyResultRequest verifyResultRequest = new DescribeVerifyResultRequest();
            verifyResultRequest.setBizId(bizId);
            verifyResultRequest.setBizType(bizType);
            DescribeVerifyResultResponse verifyResultResponse = client.getAcsResponse(verifyResultRequest);
            log.info("阿里实人认证结果：{}", JSON.toJSONString(verifyResultResponse));
            if (verifyResultResponse != null && verifyResultResponse.getVerifyStatus() == 1) {
                return true;
            }
        } catch (Exception e) {
            log.info("获取阿里实人认证出错", e);
        }
        return false;
    }

    /**
     * 获取阿里人脸对比相似度
     *
     * @param channelId
     * @param sourceImageType
     * @param sourceImgValue
     * @param targetImageType
     * @param targetImageValue
     * @return
     */
    @Override
    public Float getAliCompareFace(Long channelId, ImgTypeEnum sourceImageType, String sourceImgValue, ImgTypeEnum targetImageType, String targetImageValue) {
        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", "AccessKey", "AccessSecret");
        IAcsClient client = new DefaultAcsClient(profile);
        CompareFacesRequest request = new CompareFacesRequest();
        request.setMethod(MethodType.POST);
        request.setSourceImageType(sourceImageType.getCode());
        request.setSourceImageValue(sourceImgValue);
        request.setTargetImageType(targetImageType.getCode());
        request.setTargetImageValue(targetImageValue);
        try {
            CompareFacesResponse response = client.getAcsResponse(request);
            if (response != null && response.getSuccess()) {
                return response.getData().getSimilarityScore();
            }
        } catch (Exception e) {
            log.info("调用阿里人脸对比出错", e);
        }
        return null;
    }

    /**
     * jsonp转json
     *
     * @param jsonp
     * @return
     */
    public static String parseJSONP(String jsonp) {
        int startIndex = jsonp.indexOf("(");
        int endIndex = jsonp.lastIndexOf(")");
        String json = jsonp.substring(startIndex + 1, endIndex);
        return json;
    }

    public static void main(String[] args) {
        String resp = "callback( {\"error\":100005,\"error_description\":\"param code is wrong or lost \"} );\n";
        String s = parseJSONP(resp);
        System.out.println(s);
    }
}
