package com.starbuds.server.service.user.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.db.UserThirdPartyHisDO;
import com.starbuds.server.common.pojo.user.NoticeUserRegistered;
import com.starbuds.server.common.pojo.user.NoticeUserUpdate;
import com.starbuds.server.common.pojo.user.ThirdPartyUser;
import com.starbuds.server.common.service.user.ThirdPartyService;
import com.starbuds.server.service.user.dao.ChannelDAO;
import com.starbuds.server.service.user.dao.ChannelSecretDAO;
import com.starbuds.server.service.user.dao.UserThirdPartyHisDAO;
import com.starbuds.server.service.user.pojo.ThirdPartyNoticeApi;
import com.starbuds.server.service.user.request.NoticeUserRegisteredReq;
import com.starbuds.server.service.user.request.NoticeUserUpdateReq;
import com.starbuds.server.service.user.request.UserReq;
import com.starbuds.server.service.user.response.BodyRes;
import com.starbuds.server.service.user.response.UserRes;
import com.starbuds.server.service.user.util.UserSignUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.validator.routines.UrlValidator;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 第三方服务实现
 *
 * @author aoe
 * @date 2020/6/5
 */
@Service(version = "1.0.0")
@Slf4j
public class ThirdPartyServiceImpl implements ThirdPartyService {

    @Resource
    private ChannelDAO channelDAO;

    @Resource
    private ChannelSecretDAO channelSecretDAO;

    @Resource
    private UserThirdPartyHisDAO userThirdPartyHisDAO;

    @Override
    public ThirdPartyUser getUserByPhone(Long channelId, String areaCode, String phone) {
        if (channelId == null || StringUtils.isBlank(phone)) {
            log.warn("第三方用户信息channelId、phone 有空值，不能发起请求 channelId={} phone={}", channelId, phone);
            return null;
        }
        String sceretKey = channelSecretDAO.selectSecertKey(channelId);
        if (StringUtils.isBlank(sceretKey)) {
            log.warn("第三方用户信息没有配置秘钥，不能发起请求 channelId={}", channelId);
            return null;
        }
        areaCode = StringUtils.isBlank(areaCode) ? "86" : areaCode;

        // 第三方用户信息
        UserRes user3Res = getUserResByPhone(sceretKey, channelId, areaCode, phone);
        if (user3Res == null) {
            return null;
        }
        ThirdPartyUser user3 = new ThirdPartyUser();
        BeanUtils.copyProperties(user3Res, user3);
        user3.setNickName(user3Res.getNickname());
        user3.setHeadImgUrl(user3Res.getUserAvatar());

        try {
            Date bornTime = DateUtil.parse(user3Res.getBornTime(), "yyyy-MM-dd");
            user3.setUserBornTime(bornTime.getTime());
        } catch (Exception e) {
            log.error(StrUtil.format("用户生日格式错误 bornTime={} phone={}", user3Res.getBornTime(), phone), e);
        }

        // TODO 国家暂时都是中国 tb_data_country
        user3.setCountryId(1);
        // TODO 地区暂时为空
        user3.setAreaId(null);

        return user3;
    }

    @Override
    public void sendNoticeUserRegistered(NoticeUserRegistered userRegistered) {
        ThirdPartyNoticeApi notice = channelDAO.selectByChannelId(userRegistered.getChannelId());
        if (notice == null) {
            return;
        }

        String url = notice.getApiNoticeUserRegistered();
        boolean isValid = checkUrl(url);
        if (!isValid) {
            return;
        }
        String sceretKey = notice.getSecretKey();
        if (StringUtils.isBlank(sceretKey)) {
            return;
        }

        NoticeUserRegisteredReq req = new NoticeUserRegisteredReq();
        BeanUtils.copyProperties(userRegistered, req);
        req.setPhone(userRegistered.getUserPhone());
        req.setNickname(userRegistered.getUserName());
        req.setTimestamp(System.currentTimeMillis());
        req.setSign(getSignStr(sceretKey, req));

        String body = JSON.toJSONString(req);
        String res = null;
        try {
            res = HttpUtil.post(url, body, 2000);
        } catch (Exception e) {
            log.error("出现错误", e);
        }
        log.info("星芽注册用户通知第三方 url={} body={} res={}", url, body, res);
    }

    @Override
    public void sendNoticeUserUpdate(NoticeUserUpdate userUpdate) {
        log.info("接收到星芽用户修改信息通知第三方信息 userUpdate={}", userUpdate);
        List<ThirdPartyNoticeApi> noticeApiList = channelDAO.selectThirdPartyNoticeApiList();
        if (CollectionUtils.isEmpty(noticeApiList)) {
            log.info("第三方用户通知类API信息列表为空，不需要通知");
            return;
        }

        for (ThirdPartyNoticeApi notice : noticeApiList) {
            if (notice == null) {
                continue;
            }
            Long channelId = notice.getChannelId();
            String url = notice.getApiNoticeUserUpdate();
            boolean isValid = checkUrl(url);
            if (!isValid) {
                continue;
            }
            String sceretKey = channelSecretDAO.selectSecertKey(channelId);
            if (StringUtils.isBlank(sceretKey)) {
                log.info("无效的秘钥，不发送通知, channelId={} 秘钥={}", channelId, sceretKey);
                continue;
            }

            NoticeUserUpdateReq req = new NoticeUserUpdateReq();
            BeanUtils.copyProperties(userUpdate, req);
            req.setNickname(userUpdate.getUserName());
            req.setTimestamp(System.currentTimeMillis());
            req.setSign(getSignStr(sceretKey, req));

            String body = JSON.toJSONString(req);
            String res = null;
            try {
                res = HttpUtil.post(url, body, 2000);
            } catch (Exception e) {
                log.error("出现错误", e);
            }
            log.info("星芽用户修改信息通知第三方 url={} body={} res={}", url, body, res);
        }
    }

    private UserRes getUserResByPhone(String sceretKey, Long channelId, String areaCode, String phone) {
        // 获取第三方登陆信息
        String url = channelDAO.selectApiAuthByPhone(channelId);
        boolean isValid = checkUrl(url);
        if (!isValid) {
            return null;
        }

        UserReq body = new UserReq();
        body.setCountry(areaCode);
        body.setPhone(phone);
        body.setTimestamp(System.currentTimeMillis());
        body.setSign(getSignStr(sceretKey, body));
        String bodyJson = JSON.toJSONString(body);

        String resStr = HttpUtil.post(url, bodyJson, 2000);
        log.info("获取第三方用户信息 url={} body={} res={}", url, bodyJson, resStr);

        // 记录返回结果
        UserThirdPartyHisDO his = new UserThirdPartyHisDO();
        his.setUrl(url);
        his.setResponse(resStr);
        his.setCreateTime(System.currentTimeMillis());
        // 类型：获取第三方登陆信息（目前只有一个类型）
        his.setSendType(10);
        int count = userThirdPartyHisDAO.insert(his);
        log.info("记录获取第三方登陆信息结果 count={} his={}", count, his);

        if (!JSON.isValid(resStr)) {
            log.error("获取第三方登陆信息返回结果格式错误 url={} body={} res={}", url, body, resStr);
            return null;
        }

        BodyRes res = JSON.parseObject(resStr, BodyRes.class);
        if (res == null && res.getCode() != null || !"OK".equals(res.getCode().toUpperCase())) {
            log.error("获取第三方登陆信息返回结果无效 url={} body={} res={}", url, body, resStr);
        }

        return res.getData();
    }

    /**
     * 验证Url是否有效
     *
     * @param url
     * @return true 有效
     */
    boolean checkUrl(String url) {
        String[] schemes = {"http", "https"};
        UrlValidator urlValidator = new UrlValidator(schemes);
        return urlValidator.isValid(url);
    }

    /**
     * 获得APP手机登录接口的签名
     *
     * @param secretKey
     * @param country
     * @param phone
     * @return
     */
    private String getSignUserByPhone(String secretKey, String country, String phone) {
        String sign = StrUtil.format("country={}&phone={}", country, phone);
        sign += secretKey;
        return SecureUtil.md5(sign);
    }

    /**
     * 获得签名字符串
     *
     * @param secretKey
     * @param obj
     * @return
     */
    private String getSignStr(String secretKey, Object obj) {
        String jsonStr = JSON.toJSONString(obj);
        JSONObject json = JSON.parseObject(jsonStr);
        Map<String, Object> params = new HashMap<>();
        for (Map.Entry entry : json.entrySet()) {
            params.put(entry.getKey() + "", entry.getValue());
        }
        return UserSignUtil.md5Sign(params, secretKey);
    }

    /**
     * 签名测试
     *
     * @param str
     * @return
     */
    private static String signTest(String str) {
        return SecureUtil.md5(str).toUpperCase();
    }

    public static void main(String[] args) {
        System.out.println(signTest("country=86&phone=18001987623&timestamp=1591190480051&key=hKNAPdTt4K94973qM6uoL81VmCkWxqL6"));
    }

}
