package com.starbuds.server.service.user.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.user.UserMedalTypeEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.data.MedalShinyStatus;
import com.starbuds.server.common.pojo.db.UserMedalShinyStatusDO;
import com.starbuds.server.common.pojo.message.im.rtc.UserMedalShinyStatusChangedMsg;
import com.starbuds.server.common.pojo.user.UserSimpleVO;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.message.biz.UserMedalMsgService;
import com.starbuds.server.common.service.user.UserMedalJobService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.user.dao.UserMedalShinyStatusDAO;
import com.starbuds.server.service.user.provider.UserMedalStatusProvider;
import com.starbuds.server.service.user.provider.impl.UserMedalShinyDate;
import com.starbuds.server.service.user.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * 用户勋章定时任务
 *
 * @author aoe
 * @date 2022/1/17
 */
@Slf4j
@Service(version = "1.0.0")
public class UserMedalJobServiceImpl implements UserMedalJobService {

    @Resource
    private UserMedalShinyStatusDAO userMedalShinyStatusDAO;

    @Resource
    private UserMedalShinyDate userMedalShinyDate;

    @Reference(version = "1.0.0")
    private UserMedalMsgService userMedalMsgService;

    @Reference(version = "1.0.0")
    private ImService imService;

    @Resource
    private UserService userService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private UserMedalStatusProvider userMedalStatusProvider;

    @Override
    public int sendPutGreyMessages() {
        UserMedalTypeEnum wealthLevel = UserMedalTypeEnum.WEALTH_LEVEL;
        List<Long> sendPutGreyMessageUserIds = getSendPutGreyMessageUserIds(wealthLevel);
        if (CollectionUtils.isEmpty(sendPutGreyMessageUserIds)) {
            return 0;
        }
        int sendCount = updateSendMsgStatusToHasBeenSent(sendPutGreyMessageUserIds, wealthLevel);
        sendMedalStatusToGreyImMessages(sendPutGreyMessageUserIds);
        sendMedalStatusToGreySystemMessages(sendPutGreyMessageUserIds);
        cleanMedalStatusCache(sendPutGreyMessageUserIds);
        cleanUsersCacheInRoom(sendPutGreyMessageUserIds);
        log.info("勋章状态置灰成功 userIds={}", JSON.toJSONString(sendPutGreyMessageUserIds));
        return sendCount;
    }

    private List<Long> getSendPutGreyMessageUserIds(UserMedalTypeEnum userMedalType) {
        LocalDate now = LocalDate.now();
        int today = userMedalShinyDate.getToday(now);
        List<Long> userIds = userMedalShinyStatusDAO.selectSendPutGreyMessageUserIds(userMedalType.value, today, 100);
        if (isDev()) {
            log.info("查询置灰勋章用户id today={} userIds={}", today, JSON.toJSONString(userIds));
        }
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }
        return userIds;
    }

    private int updateSendMsgStatusToHasBeenSent(Collection<Long> userIds, UserMedalTypeEnum userMedalType) {
        QueryWrapper qw = new QueryWrapper();
        qw.in("userId", userIds);
        qw.eq("userMedalType", userMedalType.value);

        UserMedalShinyStatusDO po = new UserMedalShinyStatusDO();
        po.setSendMsgStatus(BoolIntVal.True);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setQw(qw);
        uw.setPo(po);
        return userMedalShinyStatusDAO.update(uw);
    }

    private void sendMedalStatusToGreyImMessages(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return;
        }
        userIds.stream()
                .filter(Objects::nonNull)
                .forEach(userId -> sendMedalStatusToGreyImMsg(userId));
    }

    private boolean sendMedalStatusToGreyImMsg(Long userId) {
        UserMedalShinyStatusChangedMsg msg = new UserMedalShinyStatusChangedMsg();
        msg.setUserId(userId);

        MedalShinyStatus medalShinyStatus = new MedalShinyStatus();
        medalShinyStatus.setWealthLevel(BoolIntVal.False);
        msg.setMedalShinyStatus(medalShinyStatus);
        return userMedalMsgService.sendMedalShinyStatusChangedMsg(msg);
    }

    private void sendMedalStatusToGreySystemMessages(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return;
        }
        Map<Long, UserSimpleVO> userMap = userService.getSimpleUserMap(userIds);
        userIds.stream()
                .filter(Objects::nonNull)
                .forEach(userId -> {
                    UserSimpleVO user = userMap.get(userId);
                    if (user == null) {
                        return;
                    }
                    sendMedalStatusToGreySystemMsg(userId, user.getUserName());
                });
    }

    private void sendMedalStatusToGreySystemMsg(Long userId, String userName) {
        String content = String.format("%s，因您长时间没有礼物赠送的行为，您的用户等级勋章已置灰显示，赠送任意有价值的礼物可恢复点亮的勋章哦~", userName);
        imService.sendNoActionSystemMsg(userId, "等级勋章", content);
    }

    private void cleanMedalStatusCache(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return;
        }
        Collection<String> keys = userIds.stream()
                .filter(Objects::nonNull)
                .map(userId -> RedisUserKeys.getTodayMedalShinyStatus(userId))
                .collect(Collectors.toList());
        redisUtil.delete(keys);
    }

    private void cleanUsersCacheInRoom(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return;
        }
        userIds.stream()
                .filter(Objects::nonNull)
                .forEach(userId -> userMedalStatusProvider.cleanUserCacheInRoom(userId));
    }
}
