package com.starbuds.server.service.user.impl;

import com.starbuds.server.common.constant.cache.RedisActivityKeys;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.finance.BagItemTypeEnum;
import com.starbuds.server.common.constant.finance.IssueGoldCoinOrderTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.UserSignInConfigDO;
import com.starbuds.server.common.pojo.db.UserSignInHisDO;
import com.starbuds.server.common.pojo.db.UserSignInToastHisDO;
import com.starbuds.server.common.pojo.finance.BagItemAddDTO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.finance.BagService;
import com.starbuds.server.common.service.finance.IssueGoldCoinHisService;
import com.starbuds.server.common.service.user.UserSignInService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.user.dao.UserSignInConfigDAO;
import com.starbuds.server.service.user.dao.UserSignInHisDAO;
import com.starbuds.server.service.user.dao.UserSignInToastHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @Author: lujiao
 * @Date: 2021/8/16 10:47
 */
@Service(version = "1.0.0")
@Slf4j
public class UserSignInServiceImpl implements UserSignInService {

    @Resource
    private UserSignInConfigDAO userSignInConfigDAO;

    @Resource
    private UserSignInHisDAO userSignInHisDAO;

    @Resource
    private UserSignInToastHisDAO userSignInToastHisDAO;

    @Reference(version = "1.0.0")
    private RedisService redisService;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Reference(version = "1.0.0")
    BagService bagService;

    @Reference(version = "1.0.0")
    private IssueGoldCoinHisService issueGoldCoinHisService;

    private static final Integer REDIS_LOCK = 10;

    private static final Integer STEP_END = 7;

    /**
     * 用户签到
     *
     * @param userId
     * @param channelId
     * @return
     */
    @Override
    public Result userSignIn(Long userId, Long channelId) {
        log.info("service-user-userSignIn开始");
        String key = RedisActivityKeys.getSignInKey(userId);
        if (!redisService.tryLock(key, "", REDIS_LOCK, TimeUnit.SECONDS)) {
            return new Result(GlobalApiError.OperationFailed, "签到时间间隔过短");
        }
        UserSignInHisDO lastOne = userSignInHisDAO.getLastSignHis(userId);
        boolean cannotSignIn = true;
        // 判断今天是否已经签到了
        if (lastOne != null && lastOne.getHisId() > 0) {
            Long dayBegin = DateUtil.getDayBegin();
            if (lastOne.getCreateTime() > dayBegin) {
                cannotSignIn = false;
            }
        }
        if (cannotSignIn) {
            // 获取连续签到的天数
            Integer step = getStep(lastOne) + 1;
            // 开始签到
            UserSignInConfigDO signInConfigDO = userSignInConfigDAO.getSignConfigBySort(step);
            if (signInConfigDO == null) {
                return new Result(GlobalApiError.OperationFailed, "签到失败, 请重试");
            }
            // 记录签到历史并发放物资金币、礼物
            doUserSignIn(step, userId, signInConfigDO, channelId);
            log.info("service-user-userSignIn结束");
            return new Result();

        } else {
            return new Result(GlobalApiError.OperationFailed, "已签到");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void doUserSignIn(int step, Long userId, UserSignInConfigDO configDO, Long channelId) {
        Long hisId = identityService.getId();
        UserSignInHisDO userSignInHisDO = new UserSignInHisDO();
        userSignInHisDO.setHisId(hisId);
        userSignInHisDO.setUserId(userId);
        userSignInHisDO.setStep(step);
        userSignInHisDO.setRewardType(configDO.getRewardType());
        userSignInHisDO.setRewardId(configDO.getRewardId());
        userSignInHisDO.setRewardPrice(configDO.getRewardPrice() * configDO.getRewardNum());
        userSignInHisDO.setRewardNum(configDO.getRewardNum());
        userSignInHisDO.setCreateTime(System.currentTimeMillis());
        userSignInHisDO.setCreateDate(DateUtil.getIntDate());
        userSignInHisDO.setChannelId(channelId);
        userSignInHisDAO.insert(userSignInHisDO);

        // 开始发放物资金币、礼物等
        if (configDO.getRewardType() == BagItemTypeEnum.GOLDEN_COIN.getValue()) {
            // 这里发放金币奖励
            issueGoldCoinHisService.payGoldCoin(userId, userSignInHisDO.getRewardNum(), hisId,
                    IssueGoldCoinOrderTypeEnum.DAILY_SIGN_IN, String.format("连续签到%s天", step));

        } else {
            // 这里发放礼物啥的
            List<BagItemAddDTO> dtoList = new ArrayList<>();
            Long expireTime = -1L;
            if (configDO.getDuration() > 0) {
                expireTime = System.currentTimeMillis() + configDO.getDuration();
            }
            BagItemAddDTO dto = new BagItemAddDTO(userId, configDO.getRewardId(), configDO.getRewardType(), configDO.getRewardNum(), expireTime, "签到赠送", hisId, configDO.getRewardName(), configDO.getImgUrl(), configDO.getDuration());
            dtoList.add(dto);
            bagService.batchAddItem(dtoList);
        }
    }

    /**
     * 获取当前的连续签到次数
     *
     * @param lastOne
     * @return
     */
    @Override
    public Integer getStep(UserSignInHisDO lastOne) {
        int step = 0;
        if (lastOne != null && lastOne.getHisId() > 0) {
            step = lastOne.getStep();
            // 一个周期清空
            if (step == STEP_END) {
                step = 0;
            }
            Long yesterdayBegin = DateUtil.getDayBegin() - DateUtil.DAY_OF_MILLION;
            // 如果断签了, 需要从头开始
            if (lastOne.getCreateTime() < yesterdayBegin) {
                step = 0;
            }
        }
        return step;
    }

    /**
     * 获取签到配置去重 + 排序
     *
     * @return
     */
    @Override
    public List<UserSignInConfigDO> getSignInConfig() {
        return userSignInConfigDAO.getSignConfig();
    }

    /**
     * 获取上次签到记录
     *
     * @param userId
     * @return
     */
    @Override
    public UserSignInHisDO getLastSignHis(Long userId) {
        return userSignInHisDAO.getLastSignHis(userId);
    }

    @Override
    public void addSignInToastHis(Long userId, Integer step, Long channelId) {
        UserSignInToastHisDO hisDO = new UserSignInToastHisDO();
        hisDO.setHisId(identityService.getId());
        hisDO.setUserId(userId);
        hisDO.setStep(step);
        hisDO.setChannelId(channelId);
        hisDO.setCreateDate(DateUtil.getIntDate());
        hisDO.setCreateTime(System.currentTimeMillis());
        userSignInToastHisDAO.insert(hisDO);
    }
}
