package com.starbuds.server.service.user.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.user.LiveUserRole;
import com.starbuds.server.common.pojo.db.ChatGroupDO;
import com.starbuds.server.common.pojo.user.LiveUserProfile;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.MedalService;
import com.starbuds.server.common.service.media.ChatService;
import com.starbuds.server.service.user.dao.UserDAO;
import com.starbuds.server.service.user.dao.UserSuperDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/3/23
 */
@Component
@Slf4j
public class ChatUserProvider {

    @Reference(version = "1.0.0")
    ChatService chatService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Resource
    UserDAO userDAO;

    @Resource
    UserSuperDAO userSuperDAO;

    /**
     * 获取用户直播间资料字符串
     *
     * @param userId
     * @return
     */
    public String getLiveChatUserProfileString(Long userId, String group) {
        // 用户昵称
        UserCardVO userCardVO = userDAO.selectCardById(userId);
        return getProfileString(userCardVO, group);
    }

    /**
     * 获取用户在直播间的资料
     *
     * @param userId
     * @param groupId
     * @return
     */
    public LiveUserProfile getUserProfile(Long userId, String groupId) {
        Object obj = cacheService.hashGet(RedisCacheKeys.getChatGroupUserKey(groupId), String.valueOf(userId));
        if (obj != null) {
            return JSON.parseObject(obj.toString(), LiveUserProfile.class);
        }
        return cacheChatUserProfile(userId, groupId);
    }

    /**
     * 缓存用户直播间资料
     *
     * @param userId
     * @return
     */
    public LiveUserProfile cacheChatUserProfile(Long userId, String groupId) {
        UserCardVO userCardVO = userDAO.selectCardById(userId);
        if (userCardVO != null) {
            ChatGroupDO chatGroup = chatService.getChatGroup(groupId);
            LiveUserProfile liveUserProfile = new LiveUserProfile();
            liveUserProfile.setUserId(userId);
            liveUserProfile.setProfileString(getProfileString(userCardVO, groupId));
            liveUserProfile.setUserName(userCardVO.getUserName());
            liveUserProfile.setUserAvatar(userCardVO.getUserAvatar());
            liveUserProfile.setHeadWearAnimationId(headwearService.getUserEnabledHeadwearAnimationId(userId));
            liveUserProfile.setMedalId(medalService.getEnabledMedalId(userId));
            int userRole = LiveUserRole.AUDIENCE;
            if (chatGroup != null) {
                if (userId.equals(chatGroup.getOwnerUserId())) {
                    userRole = LiveUserRole.ANCHOR;
                } else if (userSuperDAO.countById(userId) > 0) {
                    userRole = LiveUserRole.SUPER;
                }
            }
            liveUserProfile.setUserRole(userRole);

            // 缓存基本资料
            cacheService.hashPut(RedisCacheKeys.getChatGroupUserKey(groupId), String.valueOf(userId), JSON.toJSONString(liveUserProfile));

            // 缓存所在组
            cacheService.hashPut(RedisCacheKeys.CHAT_USER_GROUP, String.valueOf(userId), groupId);

            return liveUserProfile;
        }
        return null;
    }

    /**
     * 获取属性字符串
     *
     * @param userCardVO
     * @param group
     * @return
     */
    public String getProfileString(UserCardVO userCardVO, String group) {
        if (userCardVO != null) {
            StringBuilder sbProfileString = new StringBuilder();
            // 财富等级 1-3
            sbProfileString.append(fillZeroPrefix(Optional.ofNullable(userCardVO.getWealthLevel()).map(t -> t > 0 ? t : 1).orElse(1), 3));
            // 贵族等级 4
            sbProfileString.append(Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0));
            // 守护等级 5
            sbProfileString.append("0");
            // 会员等级 6
            sbProfileString.append(Optional.ofNullable(userCardVO.getVipLevel()).orElse(0));
            // 粉丝团等级 7-8
            sbProfileString.append("00");
            // 预留 9-10
            sbProfileString.append("00");
            // 11-20 靓号
            sbProfileString.append(fillZeroPrefix(Optional.ofNullable(userCardVO.getNiceNo()).orElse(0L), 10));
            return sbProfileString.toString();
        }
        return null;
    }

    /**
     * 移除用户直播间缓存资料
     *
     * @param userId
     * @return
     */
    public boolean removeUserProfile(Long userId, String groupId) {
        try {
            // 移除基本缓存
            cacheService.hashDel(RedisCacheKeys.getChatGroupUserKey(groupId), String.valueOf(userId));
            // 移除所在组
            cacheService.hashDel(RedisCacheKeys.CHAT_USER_GROUP, String.valueOf(userId));
            return true;
        } catch (Exception e) {
            log.error("用户直播间缓存资料移除失败", e);
        }
        return false;
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @param groupId
     * @return
     */
    public boolean refreshUserProfile(Long userId, String groupId) {
        if (cacheService.hashExists(RedisCacheKeys.getChatGroupUserKey(groupId), String.valueOf(userId))) {
            cacheChatUserProfile(userId, groupId);
            return true;
        }
        return false;
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @return
     */
    public boolean refreshUserProfile(Long userId) {
        String groupId = Optional.ofNullable(cacheService.hashGet(RedisCacheKeys.CHAT_USER_GROUP, String.valueOf(userId))).map(Object::toString).orElse("");
        if (StringUtils.isNotBlank(groupId)) {
            return refreshUserProfile(userId, groupId);
        }
        return false;
    }

    public static String fillZeroPrefix(long value, int num) {
        int strLen = String.valueOf(value).length();
        if (strLen >= num) {
            return String.valueOf(value);
        }
        int needNum = (int) Math.pow(10, num - strLen);
        return String.valueOf(needNum).substring(1) + value;
    }
}
