package com.starbuds.server.service.user.provider;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.user.LiveUserRole;
import com.starbuds.server.common.pojo.user.LiveUserProfile;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.MedalService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.service.user.dao.AnchorFansDAO;
import com.starbuds.server.service.user.dao.AnchorMasterDAO;
import com.starbuds.server.service.user.dao.UserDAO;
import com.starbuds.server.service.user.dao.UserSuperDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * 直播间用户服务
 *
 * @author Lay
 * @date 2020/1/10
 */
@Component
@Slf4j
public class LiveUserProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Resource
    UserDAO userDAO;

    @Resource
    AnchorMasterDAO anchorMasterDAO;

    @Resource
    AnchorFansDAO anchorFansDAO;

    @Resource
    UserSuperDAO userSuperDAO;

    /**
     * 获取用户直播间资料字符串
     *
     * @param userId
     * @return
     */
    public String getLiveChatUserProfileString(Long userId, Long anchorUserId) {
        // 用户昵称
        UserCardVO userCardVO = userDAO.selectCardById(userId);
        return getProfileString(userCardVO, anchorUserId);
    }

    /**
     * 获取用户在直播间的资料
     *
     * @param userId
     * @param anchorUserId
     * @return
     */
    public LiveUserProfile getLiveUserProfile(Long userId, Long anchorUserId) {
        Object obj = redisService.hashGet(RedisCacheKeys.getLiveUserKey(anchorUserId), String.valueOf(userId));
        if (obj != null) {
            return JSON.parseObject(obj.toString(), LiveUserProfile.class);
        }
        return cacheLiveUserProfile(userId, anchorUserId);
    }

    /**
     * 缓存用户直播间资料
     *
     * @param userId
     * @return
     */
    public LiveUserProfile cacheLiveUserProfile(Long userId, Long anchorUserId) {
        UserCardVO userCardVO = userDAO.selectCardById(userId);
        if (userCardVO != null) {
            LiveUserProfile liveUserProfile = new LiveUserProfile();
            liveUserProfile.setUserId(userId);
            liveUserProfile.setProfileString(getProfileString(userCardVO, anchorUserId));
            liveUserProfile.setUserName(userCardVO.getUserName());
            liveUserProfile.setUserAvatar(userCardVO.getUserAvatar());
            liveUserProfile.setHeadWearAnimationId(headwearService.getUserEnabledHeadwearAnimationId(userId));
            liveUserProfile.setMedalId(medalService.getEnabledMedalId(userId));
            int userRole = LiveUserRole.AUDIENCE;
            if (userId.equals(anchorUserId)) {
                userRole = LiveUserRole.ANCHOR;
            } else if (userSuperDAO.countById(userId) > 0) {
                userRole = LiveUserRole.SUPER;
            } else if (anchorMasterDAO.countById(anchorUserId, userId) > 0) {
                userRole = LiveUserRole.MASTER;
            }
            liveUserProfile.setUserRole(userRole);

            // 贵族等级
            Integer nobleLevel = Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0);

            // 粉丝团等级
            Integer fansLevel = anchorFansDAO.countById(anchorUserId, userId);

            // 缓存基本资料
            redisService.hashPut(RedisCacheKeys.getLiveUserKey(anchorUserId), String.valueOf(userId), JSON.toJSONString(liveUserProfile));

            // 缓存在线链表
            redisService.zAdd(RedisCacheKeys.getLiveOnlineKey(anchorUserId), String.valueOf(userId), getOnlineScore(nobleLevel, 0, 0, fansLevel, userCardVO.getWealthLevel()));

            // 缓存所在直播间
            redisService.hashPut(RedisCacheKeys.LIVE_USER_ROOM, String.valueOf(userId), String.valueOf(anchorUserId));

            // 缓存贵族
            if (nobleLevel > 0) {
                redisService.zAdd(RedisCacheKeys.getLiveNobleKey(anchorUserId), String.valueOf(userId), Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0));
            } else {
                redisService.zRemove(RedisCacheKeys.getLiveNobleKey(anchorUserId), String.valueOf(userId));
            }

            // 缓存粉丝团
            if (fansLevel > 0) {
                redisService.zAdd(RedisCacheKeys.getLiveFansKey(anchorUserId), String.valueOf(userId), Optional.ofNullable(userCardVO.getWealthLevel()).map(t -> Double.valueOf(t)).orElse(1d));
            }

            return liveUserProfile;
        }
        return null;
    }

    /**
     * 移除用户直播间缓存资料
     *
     * @param userId
     * @return
     */
    public boolean removeLiveUserProfile(Long userId, Long anchorUserId) {
        try {
            log.info("移除用户直播间缓存资料");

            // 移除基本缓存
            redisService.hashDel(RedisCacheKeys.getLiveUserKey(anchorUserId), String.valueOf(userId));

            // 移除在线链表
            Long remCount = redisService.zRemove(RedisCacheKeys.getLiveOnlineKey(anchorUserId), String.valueOf(userId));

            // 移除直播间
            redisService.hashDel(RedisCacheKeys.LIVE_USER_ROOM, String.valueOf(userId));

            // 移除贵族
            redisService.zRemove(RedisCacheKeys.getLiveNobleKey(anchorUserId), String.valueOf(userId));

            // 移除粉丝团
            redisService.zRemove(RedisCacheKeys.getLiveFansKey(anchorUserId), String.valueOf(userId));
            return remCount > 0;
        } catch (Exception e) {
            log.error("用户直播间缓存资料移除失败", e);
        }
        return false;
    }

    public Long getUserRoom(Long userId) {
        return Optional.ofNullable(redisService.hashGet(RedisCacheKeys.LIVE_USER_ROOM, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @param anchorUserId
     * @return
     */
    public boolean refreshLiveUserProfile(Long userId, Long anchorUserId) {
        if (redisService.hashExists(RedisCacheKeys.getLiveUserKey(anchorUserId), String.valueOf(userId))) {
            cacheLiveUserProfile(userId, anchorUserId);
            return true;
        }
        return false;
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @return
     */
    public boolean refreshLiveUserProfile(Long userId) {
        Long anchorUserId = Optional.ofNullable(redisService.hashGet(RedisCacheKeys.LIVE_USER_ROOM, String.valueOf(userId))).map(t -> Convert.toLong(t)).orElse(0L);
        if (anchorUserId > 0) {
            return refreshLiveUserProfile(userId, anchorUserId);
        }
        return false;
    }

    /**
     * 获取属性字符串
     *
     * @param userCardVO
     * @param anchorUserId
     * @return
     */
    public String getProfileString(UserCardVO userCardVO, Long anchorUserId) {
        if (userCardVO != null) {
            StringBuilder sbProfileString = new StringBuilder();
            // 财富等级 1-3
            sbProfileString.append(fillZeroPrefix(Optional.ofNullable(userCardVO.getWealthLevel()).map(t -> t > 0 ? t : 1).orElse(1), 3));
            // 贵族等级 4
            sbProfileString.append(Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0));
            // 守护等级 5
            sbProfileString.append("0");
            // 会员等级 6
            sbProfileString.append(Optional.ofNullable(userCardVO.getVipLevel()).orElse(0));
            // 粉丝团等级 7-8
            sbProfileString.append(fillZeroPrefix(anchorFansDAO.countById(anchorUserId, userCardVO.getUserId()), 2));
            // 9 是否新人
            sbProfileString.append(isNovice(userCardVO.getUserCreateTime()) ? "1" : "0");
            // 预留 10
            sbProfileString.append(Optional.ofNullable(userCardVO.getNiceLevel()).orElse(0));
            // 11-20 靓号
            sbProfileString.append(fillZeroPrefix(Optional.ofNullable(userCardVO.getNiceNo()).orElse(0L), 10));
            return sbProfileString.toString();
        }
        return null;
    }

    public boolean isSuper(Long userId) {
        return userSuperDAO.countById(userId) > 0;
    }

    public static String fillZeroPrefix(long value, int num) {
        int strLen = String.valueOf(value).length();
        if (strLen >= num) {
            return String.valueOf(value);
        }
        int needNum = (int) Math.pow(10, num - strLen);
        return String.valueOf(needNum).substring(1) + value;
    }

    /**
     * 获取在线积分
     *
     * @param nobleLevel
     * @param guardLevel
     * @param vipLevel
     * @param fansLevel
     * @param wealthLevel
     * @return
     */
    public static Double getOnlineScore(Integer nobleLevel, Integer guardLevel, Integer vipLevel, Integer fansLevel, Integer wealthLevel) {
        Double score = 0d;
        nobleLevel = Optional.ofNullable(nobleLevel).orElse(0);
        guardLevel = Optional.ofNullable(guardLevel).orElse(0);
        vipLevel = Optional.ofNullable(vipLevel).orElse(0);
        fansLevel = Optional.ofNullable(fansLevel).orElse(0);
        wealthLevel = Optional.ofNullable(wealthLevel).orElse(1);
        if (nobleLevel > 0) {
            score += nobleLevel * 100000;
        }
        if (guardLevel > 0) {
            score += guardLevel * 10000;
        }
        if (vipLevel > 0) {
            score += vipLevel * 1000;
        }
        if (fansLevel > 0) {
            score += fansLevel * 100;
        }
        score += wealthLevel;
        return score;
    }

    /**
     * 判断是否新人
     *
     * @param userCreateTime
     * @return
     */
    public static boolean isNovice(Long userCreateTime) {
        // 注册时间小于 3 天，新人
        return System.currentTimeMillis() - userCreateTime < 3 * TimeConstants.DAY_MILLS;
    }
}
