package com.starbuds.server.service.user.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.pojo.data.MedalShinyStatus;
import com.starbuds.server.common.pojo.user.LiveUserProfile;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.MedalService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.ScoreService;
import com.starbuds.server.common.service.user.UserMedalService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.user.dao.AnchorFansDAO;
import com.starbuds.server.service.user.dao.UserDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 直播间用户服务
 *
 * @author Lay
 * @date 2020/1/10
 */
@Component
@Slf4j
public class RtcUserProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    @Reference(version = "1.0.0")
    private MedalService medalService;

    @Reference(version = "1.0.0")
    private ScoreService scoreService;

    @Resource
    UserDAO userDAO;

    @Resource
    AnchorFansDAO anchorFansDAO;

    @Resource
    private UserMedalService userMedalService;

    /**
     * 获取用户直播间资料字符串
     *
     * @param userId
     * @return
     */
    public String getRtcUserProfileString(Long userId, Long roomId) {
        // 用户昵称
        UserCardVO userCardVO = userDAO.selectCardById(userId);
        return getProfileString(userCardVO, roomId);
    }

    /**
     * 获取用户在直播间的资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    public LiveUserProfile getRtcUserProfile(Long userId, Long roomId) {
        Object obj = redisService.hashGet(RedisRtcKeys.getRtcRoomUser(roomId), String.valueOf(userId));
        if (obj != null) {
            return JSON.parseObject(obj.toString(), LiveUserProfile.class);
        }

        UserCardVO userCardVO = userDAO.selectCardById(userId);
        return getProfile(userCardVO, roomId);
    }

    public List<LiveUserProfile> getRtcUserProfiles(Long roomId) {
        if (roomId == null) {
            return new ArrayList<>();
        }

        Map<Object, Object> redisHashMap = redisService.hashGetAll(RedisRtcKeys.getRtcRoomUser(roomId));
        if (redisHashMap != null && CollectionUtil.isNotEmpty(redisHashMap.values())) {
            List<LiveUserProfile> userProfiles = redisHashMap.values()
                    .stream()
                    .map(t -> JSON.parseObject(t.toString(), LiveUserProfile.class))
                    .collect(Collectors.toList());
            List<Long> userIds = userProfiles.stream()
                    .map(LiveUserProfile::getUserId)
                    .collect(Collectors.toList());
            // 官方账号map
            Map<Long, Boolean> officialMap = userService.getOfficialMap(userIds);
            if (CollectionUtil.isEmpty(officialMap)) {
                return userProfiles;
            }
            return userProfiles.stream().filter(e -> !officialMap.get(e.getUserId())).collect(Collectors.toList());
        }

        return Collections.emptyList();
    }

    public List<Long> getRtcUserIds(Long roomId) {
        Set<Object> objects = redisService.hashKeys(RedisRtcKeys.getRtcRoomUser(roomId));
        if (objects == null) {
            return new ArrayList<>();
        }
        return objects.stream().map(Convert::toLong).collect(Collectors.toList());
    }

    private LiveUserProfile getProfile(UserCardVO userCardVO, Long roomId) {
        if (userCardVO == null) {
            return null;
        }
        log.info("查找直播间用户信息-开始");
        LiveUserProfile liveUserProfile = new LiveUserProfile();
        Long userId = userCardVO.getUserId();
        liveUserProfile.setUserId(userId);
        liveUserProfile.setProfileString(getProfileString(userCardVO, roomId));
        liveUserProfile.setUserName(userCardVO.getUserName());
        liveUserProfile.setUserAvatar(userCardVO.getUserAvatar());
        log.info("查找直播间用户信息-中1");
        liveUserProfile.setHeadWearAnimationId(headwearService.getUserEnabledHeadwearAnimationId(userId));
        log.info("查找直播间用户信息-中2");
        liveUserProfile.setMedalId(medalService.getEnabledMedalId(userId));
        liveUserProfile.setUserNo(userCardVO.getUserNo());
        liveUserProfile.setActivitiesMedalIdArr(scoreService.getScoreLevelIds(userId));
        log.info("查找直播间用户信息-中3");
        int userRole = rtcRoomService.getRoomRole(roomId, userId);
        log.info("查找直播间用户信息-中4");
        liveUserProfile.setUserRole(userRole);

        MedalShinyStatus medalShinyStatus = userMedalService.getMedalShinyStatus(userId);
        liveUserProfile.setMedalShinyStatus(medalShinyStatus);
        log.info("查找直播间用户信息-结束");
        return liveUserProfile;
    }

    /**
     * 缓存用户直播间资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    public LiveUserProfile cacheRtcUserProfile(Long userId, Long roomId) {
        UserCardVO userCardVO = userDAO.selectCardById(userId);
        LiveUserProfile liveUserProfile = getProfile(userCardVO, roomId);
        if (liveUserProfile != null) {
            // 贵族等级
            Integer nobleLevel = Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0);

            // 会员等级
            Integer vipLevel = Optional.ofNullable(userCardVO.getVipLevel()).orElse(0);

            // 缓存基本资料
            redisService.hashPut(RedisRtcKeys.getRtcRoomUser(roomId), String.valueOf(userId), JSON.toJSONString(liveUserProfile));

            // 缓存在线链表
            redisService.zAdd(RedisRtcKeys.getRtcRoomOnline(roomId), String.valueOf(userId), getOnlineScore(nobleLevel, 0, vipLevel, 0, userCardVO.getWealthLevel()));

            // 缓存所在直播间
            redisService.hashPut(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId), String.valueOf(roomId));

            // 缓存贵族
            if (nobleLevel > 0) {
                redisService.zAdd(RedisRtcKeys.getRtcRoomNoble(roomId), String.valueOf(userId), Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0));
            }

            return liveUserProfile;
        }

        return null;
    }

    public boolean isInRoom(Long roomId, Long userId) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId))).map(Convert::toLong).orElse(0L).equals(roomId);
    }

    public Long getUserRoomId(Long userId) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
    }

    /**
     * 获得用户房间Map
     *
     * @param userIds 用户id集合
     * @return userId:roomId
     */
    public Map<Long, Long> getUserRoomIdMap(Collection<Long> userIds) {
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Object, Object> roomMap = redisService.hashGetAll(RedisRtcKeys.RTC_USER_ROOM);
        if (CollectionUtils.isEmpty(roomMap)) {
            return Collections.EMPTY_MAP;
        }

        Map<Long, Long> result = new HashMap<>();
        userIds.stream()
                .filter(userId -> userId != null)
                .forEach(userId -> {
                    Object roomId = roomMap.get(userId.toString());
                    if (roomId == null) {
                        return;
                    }
                    String roomIdStr = roomId.toString();
                    if (!NumberUtil.isLong(roomIdStr)) {
                        return;
                    }
                    result.put(userId, Long.parseLong(roomIdStr));
                });
        return result;
    }

    /**
     * 移除用户直播间缓存资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    public boolean removeRtcUserProfile(Long userId, Long roomId) {
        try {
            log.info("移除用户直播间缓存资料");

            // 移除在线链表
            Long remCount = redisService.zRemove(RedisRtcKeys.getRtcRoomOnline(roomId), String.valueOf(userId));

            if (remCount > 0) {
                // 移除基本缓存
                redisService.hashDel(RedisRtcKeys.getRtcRoomUser(roomId), String.valueOf(userId));

                // 移除直播间
                redisService.hashDel(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId));

                // 移除贵族
                redisService.zRemove(RedisRtcKeys.getRtcRoomNoble(roomId), String.valueOf(userId));
            }

            return remCount > 0;
        } catch (Exception e) {
            log.error("用户直播间缓存资料移除失败", e);
        }
        return false;
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @param roomId
     * @return
     */
    public boolean refreshRtcUserProfile(Long userId, Long roomId) {
        if (redisService.hashExists(RedisRtcKeys.getRtcRoomUser(roomId), String.valueOf(userId))) {
            cacheRtcUserProfile(userId, roomId);
            return true;
        }
        return false;
    }

    /**
     * 刷新用户直播间资料
     *
     * @param userId
     * @return
     */
    public boolean refreshRtcUserProfile(Long userId) {
        Long roomId = Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_USER_ROOM, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        if (roomId > 0) {
            return refreshRtcUserProfile(userId, roomId);
        }
        return false;
    }

    /**
     * 获取属性字符串
     *
     * @param userCardVO
     * @param anchorUserId
     * @return
     */
    public String getProfileString(UserCardVO userCardVO, Long anchorUserId) {
        if (userCardVO != null) {
            StringBuilder sbProfileString = new StringBuilder();
            // 财富等级 1-3
            sbProfileString.append(fillZeroPrefix(Optional.ofNullable(userCardVO.getWealthLevel()).map(t -> t > 0 ? t : 1).orElse(1), 3));
            // 贵族等级 4
            sbProfileString.append(Optional.ofNullable(userCardVO.getNobleLevel()).orElse(0));
            // 守护等级 5
            sbProfileString.append("0");
            // 会员等级 6
            sbProfileString.append(Optional.ofNullable(userCardVO.getVipLevel()).orElse(0));
            // 粉丝团等级 7-8
            sbProfileString.append(fillZeroPrefix(anchorFansDAO.countById(anchorUserId, userCardVO.getUserId()), 2));
            // 9 是否新人
            sbProfileString.append(isNovice(userCardVO.getUserCreateTime()) ? "1" : "0");
            // 预留 10  靓号等级
            sbProfileString.append(Optional.ofNullable(userCardVO.getNiceLevel()).orElse(0));
            // 11-20 靓号
            sbProfileString.append(fillZeroPrefix(Optional.ofNullable(userCardVO.getNiceNo()).orElse(0L), 10));
            return sbProfileString.toString();
        }
        return null;
    }

    public static String fillZeroPrefix(long value, int num) {
        int strLen = String.valueOf(value).length();
        if (strLen >= num) {
            return String.valueOf(value);
        }
        int needNum = (int) Math.pow(10, num - strLen);
        return String.valueOf(needNum).substring(1) + value;
    }

    /**
     * 获取在线积分
     *
     * @param nobleLevel
     * @param guardLevel
     * @param vipLevel
     * @param fansLevel
     * @param wealthLevel
     * @return
     */
    public static Double getOnlineScore(Integer nobleLevel, Integer guardLevel, Integer vipLevel, Integer fansLevel, Integer wealthLevel) {
        Double score = 0d;
        nobleLevel = Optional.ofNullable(nobleLevel).orElse(0);
        guardLevel = Optional.ofNullable(guardLevel).orElse(0);
        vipLevel = Optional.ofNullable(vipLevel).orElse(0);
        fansLevel = Optional.ofNullable(fansLevel).orElse(0);
        wealthLevel = Optional.ofNullable(wealthLevel).orElse(1);
        if (nobleLevel > 0) {
            score += nobleLevel * 100000;
        }
        if (guardLevel > 0) {
            score += guardLevel * 10000;
        }
        if (vipLevel > 0) {
            score += vipLevel * 1000;
        }
        if (fansLevel > 0) {
            score += fansLevel * 100;
        }
        score += wealthLevel;
        return score;
    }

    /**
     * 判断是否新人
     *
     * @param userCreateTime
     * @return
     */
    public static boolean isNovice(Long userCreateTime) {
        // 注册时间小于 3 天，新人
        return System.currentTimeMillis() - userCreateTime < 3 * TimeConstants.DAY_MILLS;
    }
}
