package com.starbuds.server.service.user.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.cache.RedisLiveKeys;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.pojo.activity.MedalPersonRankVO;
import com.starbuds.server.common.pojo.db.DataLevelScoreDO;
import com.starbuds.server.common.pojo.db.UserActivitiesLevelHisDO;
import com.starbuds.server.common.pojo.db.UserScoreDO;
import com.starbuds.server.common.pojo.db.UserScoreHisDO;
import com.starbuds.server.common.pojo.message.im.live.AnchorScoreChangeMsg;
import com.starbuds.server.common.pojo.message.im.user.ScoreChangeMsg;
import com.starbuds.server.common.pojo.mq.QueueScoreLevelUpMsg;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.activity.UserActivitiesLevelHisService;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.user.dao.UserDAO;
import com.starbuds.server.service.user.dao.UserScoreDAO;
import com.starbuds.server.service.user.dao.UserScoreHisDAO;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * @Author jiangxiaowei
 * @DateTime 2019/12/25 11:02
 * @Description 积分账户组件
 */
@Slf4j
@Component
public class ScoreProvider implements IScoreProvider {

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    UserActivitiesLevelHisService userActivitiesLevelHisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    UserScoreDAO userScoreDAO;

    @Resource
    UserScoreHisDAO userScoreHisDAO;

    @Resource
    UserDAO userDAO;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    ScoreRankProvider scoreRankProvider;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Resource
    TransactionDefinition transactionDefinition;

    public static String getScoreId(Long userId, Integer scoreType) {
        return SecureUtil.md5(String.format("%d:%d", userId, scoreType));
    }

    /**
     * 增加积分
     *
     * @param userId
     * @param scoreType
     * @param changeValue
     * @param remark
     * @param businessId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean addScore(Long userId, ScoreTypeEnum scoreType, long changeValue, String remark, Long businessId, Long anchorUserId) {
        if (isDev() && ScoreTypeEnum.CHARM == scoreType) {
            log.info(LogUtil.devLog("增加主播魅力值 接收人userId={} value={}"), userId, changeValue);
        }
        if (scoreType.getValue() == 41 || scoreType.getValue() == 42) {
            log.info("活动积分增加: userId:{},scoreType:{},changeValue:{},remark:{},businessId:{}, anchorUserId:{}", userId, scoreType.getValue(), changeValue, remark, businessId, anchorUserId);
        }
        if (changeValue < 1) {
            return false;
        }

        Long curTime = System.currentTimeMillis();

        QueueScoreLevelUpMsg levelUpMsg;
        String scoreId = getScoreId(userId, scoreType.getValue());

        // 判断账户是否存在
        UserScoreDO userScoreDO = userScoreDAO.selectById(scoreId);

        if (userScoreDO == null) {
            userScoreDO = new UserScoreDO();
            userScoreDO.setScoreId(scoreId);
            userScoreDO.setUserId(userId);
            userScoreDO.setScoreType(scoreType.getValue());
            if (ScoreTypeEnum.GIFT_RECEIVE_21.equals(scoreType) || ScoreTypeEnum.GIFT_SEND_21.equals(scoreType)) {
                userScoreDO.setScoreValue(0L);
            } else {
                userScoreDO.setScoreValue(changeValue);
            }
            userScoreDO.setVersion(0L);
            userScoreDO.setCreateTime(curTime);
            userScoreDO.setUpdateTime(curTime);

            try {
                userScoreDAO.insert(userScoreDO);
            } catch (Exception e) {
                // 重试
                return addScore(userId, scoreType, changeValue, remark, businessId, anchorUserId);
            }
        }

        Long scoreNew = executeChangeScore(userScoreDO, changeValue, remark, businessId);
        if (scoreNew == null) {
            return false;
        }

        // 排行榜
        scoreRankProvider.addRankAmount(scoreType.getValue(), userId, changeValue);

        // 等级更新判定
        levelUpMsg = handleLevelUp(scoreId, scoreType, userId, anchorUserId);

        if (ScoreTypeEnum.CHARM == scoreType) {
            // 增加主播魅力收入统计
            Long scoreValue = cacheService.hashIncreaseLong(RedisLiveKeys.LIVE_ANCHOR_CHARM, String.valueOf(userId), changeValue);

            // 给主播发送魅力值改变消息
            AnchorScoreChangeMsg anchorScoreChangeMsg = new AnchorScoreChangeMsg();
            anchorScoreChangeMsg.setScoreType(ScoreTypeEnum.CHARM.getValue());
            anchorScoreChangeMsg.setScoreValue(scoreValue);
            imService.sendLiveRoomMsg(ImTargetType.USER, null, userId, anchorScoreChangeMsg);
        }

        if (levelUpMsg != null) {
            // 发出队列消息
            rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.SCORE_LEVEL_UP, levelUpMsg.serialize());
        }

        return true;
    }


    /**
     * 减少积分
     *
     * @param userId
     * @param scoreType
     * @param changeValue
     * @param remark
     * @param businessId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long reduceScore(Long userId, ScoreTypeEnum scoreType, long changeValue, String remark, Long businessId) {
        // 取绝对值，防止参数使用错误
        changeValue = Math.abs(changeValue);

        // 判断账户是否存在
        String scoreId = getScoreId(userId, scoreType.getValue());
        UserScoreDO userScoreDO = userScoreDAO.selectById(scoreId);
        if (userScoreDO == null) {
            // 账户不存在直接返回
            return -changeValue;
        }

        return executeChangeScore(userScoreDO, -changeValue, remark, businessId);
    }

    /**
     * 执行积分改变
     *
     * @param userScoreDO
     * @param scoreValue
     * @param remarks
     * @param businessId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Long executeChangeScore(UserScoreDO userScoreDO, long scoreValue, String remarks, Long businessId) {
        try {
            // 计算新余额
            Long scoreOld = userScoreDO.getScoreValue();
            Long version = userScoreDO.getVersion();
            Long scoreNew = scoreOld + scoreValue;

            // 余额不足
            if (scoreNew < 0) {
                return scoreNew;
            }

            long curTime = System.currentTimeMillis();

            // 更新
            try {
                int updated = userScoreDAO.updateCas(userScoreDO.getScoreId(), scoreNew, version);
                if (updated == 0) {
                    userScoreDO = userScoreDAO.selectById(userScoreDO.getScoreId());
                    // 重试
                    return executeChangeScore(userScoreDO, scoreValue, remarks, businessId);
                }
            } catch (Exception e) {
                log.error("{} 更新失败，重试", userScoreDO.getScoreId());
            }

            // 写入变更记录
            UserScoreHisDO userScoreHisDO = new UserScoreHisDO();
            userScoreHisDO.setHisId(identityService.getId());
            userScoreHisDO.setScoreId(userScoreDO.getScoreId());
            userScoreHisDO.setUserId(userScoreDO.getUserId());
            userScoreHisDO.setScoreType(userScoreDO.getScoreType());
            userScoreHisDO.setChangeValue(scoreValue);
            userScoreHisDO.setCreateTime(curTime);
            userScoreHisDO.setHisRemarks(remarks);
            userScoreHisDO.setBusinessId(businessId);
            userScoreHisDAO.insert(userScoreHisDO);

            return scoreNew;
        } catch (Exception e) {
            log.error("余额变动失败", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }

        return null;
    }

    /**
     * 获取等级积分
     *
     * @param userId
     * @param scoreType
     * @return
     */
    @Override
    public Long getScoreValue(Long userId, ScoreTypeEnum scoreType) {
        return Optional.ofNullable(userScoreDAO.selectValueById(getScoreId(userId, scoreType.getValue()))).orElse(0L);
    }

    /**
     * 获取等级积分
     *
     * @param userIds
     * @param scoreType
     * @return
     */
    @Override
    public Long getScoreValues(List<Long> userIds, ScoreTypeEnum scoreType) {
        if (CollectionUtils.isEmpty(userIds) || scoreType == null) {
            return 0L;
        }
        return Optional.ofNullable(userScoreDAO.selectSumValue(userIds, scoreType.getValue())).orElse(0L);
    }

    /**
     * 获取积分等级
     *
     * @param scoreValue
     * @param scoreType
     * @return
     */
    @Override
    public int getScoreLevel(Long scoreValue, ScoreTypeEnum scoreType) {
        return Optional.ofNullable(dataService.getScoreLevel(scoreValue, scoreType)).orElse(1);
    }

    /**
     * 获取积分等级默认0
     *
     * @param scoreValue
     * @param scoreType
     * @return
     */
    @Override
    public int getScoreLevelDefault(Long scoreValue, ScoreTypeEnum scoreType) {
        return Optional.ofNullable(dataService.getScoreLevelDefault(scoreValue, scoreType)).orElse(0);
    }

    /**
     * 处理积分改变
     *
     * @param scoreId
     * @param scoreType
     * @param userId
     */
    private QueueScoreLevelUpMsg handleLevelUp(String scoreId, ScoreTypeEnum scoreType, Long userId, Long anchorUserId) {
        // 查询用户积分
        Long scoreValue = userScoreDAO.selectValueById(scoreId);

        // 新等级
        int newLevel = 1;
        if (scoreType.equals(ScoreTypeEnum.GIFT_SEND_21) || scoreType.equals(ScoreTypeEnum.GIFT_RECEIVE_21)) {
            newLevel = Optional.ofNullable(dataService.getScoreLevelDefault(BigDecimal.valueOf(scoreValue).divide(BigDecimal.valueOf(100)).longValue(), scoreType)).orElse(0);
        } else {
            newLevel = Optional.ofNullable(dataService.getScoreLevel(scoreValue, scoreType)).orElse(1);
        }

        // 当前等级
        int currentLevel = 1;

        // 积分改变消息
        ScoreChangeMsg scoreChangeMsg = new ScoreChangeMsg();
        scoreChangeMsg.setScoreType(scoreType.getValue());
        scoreChangeMsg.setScoreValue(scoreValue);
        scoreChangeMsg.setUserId(userId);

        switch (scoreType) {
            case WEALTH:
                currentLevel = Optional.ofNullable(userDAO.selectWealthLevel(userId)).orElse(1);

                scoreChangeMsg.setScoreLevel(currentLevel);
                imService.sendCustomImMsg(null, userId, scoreChangeMsg, null);

                // 如果等级变化了
                if (newLevel > currentLevel) {
                    userDAO.updateWealthLevel(userId, newLevel);

                    Integer finalLevel = newLevel;
                    try {
                        userSearchService.setWealthLevel(userId, finalLevel);
                    } catch (Exception e) {
                        log.error("更新财富值超时", e);
                    }

                    QueueScoreLevelUpMsg levelUpMsg = new QueueScoreLevelUpMsg();
                    levelUpMsg.setUserId(userId);
                    levelUpMsg.setScoreType(scoreType.getValue());
                    levelUpMsg.setScoreValue(scoreValue);
                    levelUpMsg.setScoreLevel(currentLevel);
                    levelUpMsg.setNewLevel(newLevel);
                    levelUpMsg.setAnchorUserId(anchorUserId);
                    return levelUpMsg;
                }
                break;
            case CHARM:
                currentLevel = Optional.ofNullable(userDAO.selectCharmLevel(userId)).orElse(1);

                scoreChangeMsg.setScoreLevel(currentLevel);
                imService.sendCustomImMsg(null, userId, scoreChangeMsg, null);

                // 如果等级变化了
                if (newLevel > currentLevel) {
                    userDAO.updateCharmLevel(userId, newLevel);

                    Integer finalLevel = newLevel;
                    try {
                        userSearchService.setCharmLevel(userId, finalLevel);
                    } catch (Exception e) {
                        log.error("更新魅力值超时", e);
                    }

                    QueueScoreLevelUpMsg levelUpMsg = new QueueScoreLevelUpMsg();
                    levelUpMsg.setUserId(userId);
                    levelUpMsg.setScoreType(scoreType.getValue());
                    levelUpMsg.setScoreValue(scoreValue);
                    levelUpMsg.setScoreLevel(currentLevel);
                    levelUpMsg.setNewLevel(newLevel);
                    levelUpMsg.setAnchorUserId(anchorUserId);
                    return levelUpMsg;
                }
                break;
            case STAR:
                currentLevel = Optional.ofNullable(userDAO.selectStarLevel(userId)).orElse(1);

                scoreChangeMsg.setScoreLevel(currentLevel);
                imService.sendCustomImMsg(null, userId, scoreChangeMsg, null);

                if (newLevel > currentLevel) {
                    userDAO.updateStarLevel(userId, newLevel);

                    Integer finalLevel = newLevel;
                    try {
                        userSearchService.setStarLevel(userId, finalLevel);
                    } catch (Exception e) {
                        log.error("更新星耀值超时", e);
                    }

                    QueueScoreLevelUpMsg levelUpMsg = new QueueScoreLevelUpMsg();
                    levelUpMsg.setUserId(userId);
                    levelUpMsg.setScoreType(scoreType.getValue());
                    levelUpMsg.setScoreValue(scoreValue);
                    levelUpMsg.setScoreLevel(currentLevel);
                    levelUpMsg.setNewLevel(newLevel);
                    levelUpMsg.setAnchorUserId(anchorUserId);
                    return levelUpMsg;
                }
                break;
            case GIFT_SEND_21:
            case GIFT_RECEIVE_21:
                currentLevel = Optional.ofNullable(userActivitiesLevelHisService.getUserLevel(userId, scoreType.getValue())).orElseGet(() -> {
                    UserActivitiesLevelHisDO hisDO = new UserActivitiesLevelHisDO();
                    hisDO.setLevel(0);
                    return hisDO;
                }).getLevel();


                if (ScoreTypeEnum.GIFT_SEND_21.equals(scoreType)) {
                    // 送礼成就，送到倒数第二级就不让升了.留给教皇用
                    DataLevelScoreDO lastLevel = dataService.getLastLevelScore(scoreType);
                    if (newLevel == lastLevel.getScoreLevel()) {
                        // 看能不能满足教皇条件
                        if (!isTopRankAndScoreEnough(userId)) {
                            // 不能满足就还是维持次等级
                            newLevel = currentLevel;
                        } else {
                            log.info("满足教皇条件: userId:{}", userId);
                        }
                    }
                }
                if (newLevel > currentLevel) {

                    UserActivitiesLevelHisDO hisDO = new UserActivitiesLevelHisDO();
                    hisDO.setHisId(identityService.getId());
                    hisDO.setScoreType(scoreType.getValue());
                    hisDO.setCreateTime(System.currentTimeMillis());
                    hisDO.setUserId(userId);
                    hisDO.setLevel(newLevel);
                    userActivitiesLevelHisService.create(hisDO);

                    QueueScoreLevelUpMsg levelUpMsg = new QueueScoreLevelUpMsg();
                    levelUpMsg.setUserId(userId);
                    levelUpMsg.setScoreType(scoreType.getValue());
                    levelUpMsg.setScoreValue(scoreValue);
                    levelUpMsg.setScoreLevel(currentLevel);
                    levelUpMsg.setNewLevel(newLevel);
                    levelUpMsg.setAnchorUserId(anchorUserId);
                    return levelUpMsg;
                }

                break;
            default:
                break;
        }

        return null;
    }


    /**
     * 判断是否是娱乐教皇
     *
     * @param rankSortedDesc
     * @param userId
     * @param scoreValue
     * @param scoreTopestLevel
     * @return
     */
    public boolean isTopRankAndScoreEnough(List<MedalPersonRankVO> rankSortedDesc, Long userId, Long scoreValue, DataLevelScoreDO scoreTopestLevel) {

        if (CollectionUtil.isEmpty(rankSortedDesc)) {
            return false;
        }

        for (int i = 0; i < rankSortedDesc.size(); i++) {
            MedalPersonRankVO val = rankSortedDesc.get(i);

            if (i > 2) {
                return false;
            }

            // 排名前3并且积分大于指定积分
            if (val.getUserId().equals(userId)) {
                if (scoreValue >= scoreTopestLevel.getScoreValue()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * 判断是否是娱乐教皇
     *
     * @param userId
     * @return
     */
    public boolean isTopRankAndScoreEnough(Long userId) {
        ScoreTypeEnum scoreType = ScoreTypeEnum.GIFT_SEND_21;
        Long scoreValue = getScoreValue(userId, scoreType);
        DataLevelScoreDO lastLevel = dataService.getLastLevelScore(scoreType);

        return isTopRankAndScoreEnough(getActivitiesMedalPersonRankResourceList(scoreType), userId, getRealScoreByScale(scoreValue), lastLevel);
    }

    /**
     * 获取用户积分等级排名
     *
     * @param scoreType
     * @return
     */
    public List<MedalPersonRankVO> getActivitiesMedalPersonRankResourceList(ScoreTypeEnum scoreType) {
        List<UserScoreDO> list = userScoreDAO.selectActivitiesMedalPersonRankList(scoreType.getValue());
        if (CollectionUtil.isEmpty(list)) {
            return Collections.EMPTY_LIST;
        }

        Set<Long> userIds = list.stream().map(UserScoreDO::getUserId).collect(Collectors.toSet());
        Map<Long, UserCardVO> usersMap = userService.getCardMap(userIds);

        // 获取排名
        List<MedalPersonRankVO> result = list.stream().map(val -> {
            MedalPersonRankVO vo = new MedalPersonRankVO();
            vo.setScoreId(val.getScoreId());
            vo.setScoreValue(getRealScoreByScale(val.getScoreValue()));
            if (usersMap != null) {
                UserCardVO user = Optional.ofNullable(usersMap.get(val.getUserId())).orElseGet(UserCardVO::new);
                vo.setUserAvatar(user.getUserAvatar());
                vo.setUserName(user.getUserName());
                vo.setUserId(user.getUserId());
                vo.setUserNo(user.getUserNo());
            }
            return vo;
        }).collect(Collectors.toList());

        List<MedalPersonRankVO> resultSorted = result.stream().sorted(Comparator.comparing(MedalPersonRankVO::getScoreValue).reversed()).collect(Collectors.toList());
        return resultSorted;
    }

    /**
     * 获取计算后的积分
     *
     * @param scoreValue
     * @return
     */
    private Long getRealScoreByScale(Long scoreValue) {
        return BigDecimal.valueOf(scoreValue).divide(BigDecimal.valueOf(100L)).longValue();
    }
}
