package com.starbuds.server.service.user.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.format.FastDateFormat;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.constant.user.HotLoveLeaderboardDayTypeEnum;
import com.starbuds.server.common.constant.user.ScoreRankTypeEnum;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.user.ScoreRankItem;
import com.starbuds.server.common.pojo.user.ScoreRankTimes;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.user.AnchorService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.common.constant.user.HotLoveLeaderboardDayTypeEnum.YESTERDAY;

/**
 * 积分排行
 *
 * @author Lay
 * @date 2020/2/28
 */
@Component
@Slf4j
public class ScoreRankProvider {
    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    AnchorService anchorService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    static {
        StringBuilder sb = new StringBuilder();
        // 小时榜
        sb.append("redis.call('ZINCRBY', KEYS[1], ARGV[2], ARGV[1]);");
        // 日榜
        sb.append("redis.call('ZINCRBY', KEYS[2], ARGV[2], ARGV[1]);");
        // 周榜
        sb.append("redis.call('ZINCRBY', KEYS[3], ARGV[2], ARGV[1]);");
        // 月榜
        sb.append("redis.call('ZINCRBY', KEYS[4], ARGV[2], ARGV[1]);");
        LUA_INCR = sb.toString();
    }

    static {
        StringBuilder sb = new StringBuilder();
        sb.append("local t = redis.call('TIME');");
        sb.append("return t[1]");
        LUA_TIME = sb.toString();
    }

    /**
     * 中央时间戳
     */
    public static final String LUA_TIME;

    /**
     * 加榜
     */
    public static final String LUA_INCR;

    /**
     * 获取 Redis 时间戳
     *
     * @return
     */
    private long getRedisTime() {
        return cacheService.luaLong(LUA_TIME, new ArrayList<>(), new ArrayList<>());
    }

    public ScoreRankTimes getRankTimes() {
        long redisTime = getRedisTime();
        Date date = new Date(redisTime * 1000);
        ScoreRankTimes scoreRankTimes = new ScoreRankTimes();

        // 小时
        SimpleDateFormat sdfHour = new SimpleDateFormat("yyyyMMddHH");
        scoreRankTimes.setHour(Convert.toInt(sdfHour.format(date)));

        // 日
        SimpleDateFormat sdfDay = new SimpleDateFormat("yyyyMMdd");
        scoreRankTimes.setDay(Convert.toInt(sdfDay.format(date)));

        // 基础周为 2020年1月6日第一周
        long baseTime = 1578240000;
        long secDiff = redisTime - baseTime;
        double dayDiff = secDiff / 3600 / 24;
        int week = (int) (dayDiff / 7 + 1);
        scoreRankTimes.setWeek(Convert.toInt(String.valueOf(week)));

        // 月
        SimpleDateFormat sdfMonth = new SimpleDateFormat("yyyyMM");
        scoreRankTimes.setMonth(Convert.toInt(sdfMonth.format(date)));

        return scoreRankTimes;
    }

    public void addRankAmount(Integer scoreType, Long userId, Long scoreValue) {
        ScoreRankTimes scoreRankTimes = getRankTimes();

        // 加榜
        List<String> keys = new ArrayList<>();
        keys.add(RedisCacheKeys.getScoreRankHour(scoreType, scoreRankTimes.getHour()));
        keys.add(RedisCacheKeys.getScoreRankDay(scoreType, scoreRankTimes.getDay()));
        keys.add(RedisCacheKeys.getScoreRankWeek(scoreType, scoreRankTimes.getWeek()));
        keys.add(RedisCacheKeys.getScoreRankMonth(scoreType, scoreRankTimes.getMonth()));

        // 参数
        List<String> args = new ArrayList<>();
        args.add(String.valueOf(userId));
        args.add(String.valueOf(scoreValue));

        // 加榜
        cacheService.luaLong(LUA_INCR, keys, args);
    }

    public List<ScoreRankItem> getRankItems(ScoreRankTypeEnum scoreRankType, ScoreTypeEnum scoreType, long start, long end, Long curUserId) {
        String key = null;
        ScoreRankTimes scoreRankTimes = getRankTimes();
        switch (scoreRankType) {
            case HOURLY:
                key = RedisCacheKeys.getScoreRankHour(scoreType.getValue(), scoreRankTimes.getHour());
                break;
            case DAILY:
                key = RedisCacheKeys.getScoreRankDay(scoreType.getValue(), scoreRankTimes.getDay());
                break;
            case WEEKLY:
                key = RedisCacheKeys.getScoreRankWeek(scoreType.getValue(), scoreRankTimes.getWeek());
                break;
            case MONTHLY:
                key = RedisCacheKeys.getScoreRankMonth(scoreType.getValue(), scoreRankTimes.getMonth());
                break;
            default:
                break;
        }

        if (key != null) {
            Set<Tuple> tupleSet = cacheService.zReverseRangeWithScores(key, start, end);
            if (CollectionUtil.isNotEmpty(tupleSet)) {
                List<ScoreRankItem> scoreRankItems = new ArrayList<>();
                int rankNo = Convert.toInt(start) + 1;
                Set<Long> userIds = new HashSet<>();
                List<Tuple> tupleList = new ArrayList<>(tupleSet);
                tupleList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                for (Iterator<Tuple> iterator = tupleList.iterator(); iterator.hasNext(); ) {
                    Tuple tuple = iterator.next();
                    ScoreRankItem scoreRankItem = new ScoreRankItem();
                    scoreRankItem.setRankNo(rankNo);
                    scoreRankItem.setUserId(Convert.toLong(tuple.getElement()));
                    scoreRankItem.setScoreValue(Convert.toLong(tuple.getScore()));
                    userIds.add(scoreRankItem.getUserId());
                    scoreRankItem.setOnlineStatus(userService.isOnline(scoreRankItem.getUserId()) ? BoolIntVal.True : BoolIntVal.False);
                    scoreRankItem.setFollowStatus(socialRelationService.isFollower(curUserId, scoreRankItem.getUserId()) ? BoolIntVal.True : BoolIntVal.False);
                    if (anchorService.isAnchor(scoreRankItem.getUserId())) {
                        scoreRankItem.setLiveStatus(anchorService.getLiveStatus(scoreRankItem.getUserId()).getValue());
                    }
                    scoreRankItems.add(scoreRankItem);
                    rankNo++;
                }

                // 用户数据获取
                Map<Long, UserCardVO> userCardMap = userService.getNameAndAvatarByIds(new ArrayList<>(userIds)).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
                for (ScoreRankItem scoreRankItem : scoreRankItems) {
                    UserCardVO userCard = userCardMap.get(scoreRankItem.getUserId());
                    if (userCard != null) {
                        scoreRankItem.setUserName(userCard.getUserName());
                        scoreRankItem.setUserAvatar(userCard.getUserAvatar());
                    } else {
                        scoreRankItem.setUserName("神秘人");
                    }
                }

                scoreRankItems.sort(Comparator.comparingLong(ScoreRankItem::getRankNo));
                return scoreRankItems;
            }
        }

        return Collections.emptyList();
    }

    /**
     * 增加热爱榜-每天
     *
     * @param receiverUserId 收礼物人id
     * @param senderUserId   赠送人id
     * @param diamondValue   钻石价值
     * @return
     */
    public Double addHotLoveLeaderboardDay(Long receiverUserId, Long senderUserId, Long diamondValue) {
        String today = getDay(new DateTime());
        String key = RedisCacheKeys.getHotLoveLeaderboardDay(today, receiverUserId);
        Double value = cacheService.zIncrementScore(key, senderUserId + "", diamondValue.doubleValue());
        log.info("热爱榜更新 新增钻石 {} Key {} 赠送人Id {}", diamondValue, key, senderUserId);
        return value;
    }

    /**
     * 获得热播排行榜
     *
     * @param type   -1:昨天 1:今天
     * @param userId
     * @return
     */
    public List<ScoreRankItem> getHotLoveLeaderboardList(HotLoveLeaderboardDayTypeEnum type, Long userId) {
        DateTime date = YESTERDAY == type ? DateUtil.yesterday() : new DateTime();
        String key = RedisCacheKeys.getHotLoveLeaderboardDay(getDay(date), userId);
        Set<Tuple> tupleSet = cacheService.zReverseRangeWithScores(key, 0, 30);
        if (CollectionUtil.isEmpty(tupleSet)) {
            return Collections.EMPTY_LIST;
        }

        Set<Long> userIds = new HashSet<>();
        List<ScoreRankItem> list = new ArrayList<>(tupleSet.size());
        tupleSet.stream()
                .sorted(Comparator.comparing(Tuple::getScore, Comparator.reverseOrder()))
                .forEach((tuple -> {
                    ScoreRankItem item = new ScoreRankItem();
                    item.setUserId(Long.parseLong(tuple.getElement()));
                    item.setScoreValue(tuple.getScore().longValue());
                    list.add(item);

                    userIds.add(item.getUserId());
                }));

        Map<Long, Integer> userOnLineMap = userService.isOnlineByIds(userIds);
        // 用户数据获取
        Map<Long, UserCardVO> userCardMap = userService.getNameAndAvatarByIds(userIds)
                .stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
        Integer rankNo = 0;
        for (ScoreRankItem item : list) {
            item.setRankNo(++rankNo);
            item.setOnlineStatus(userOnLineMap.get(item.getUserId()));
            item.setFollowStatus(socialRelationService.isFollower(userId, item.getUserId()) ? BoolIntVal.True : BoolIntVal.False);
            if (anchorService.isAnchor(item.getUserId())) {
                item.setLiveStatus(anchorService.getLiveStatus(item.getUserId()).getValue());
            }

            UserCardVO userCard = userCardMap.get(item.getUserId());
            if (userCard != null) {
                item.setUserName(userCard.getUserName());
                item.setUserAvatar(userCard.getUserAvatar());
            } else {
                item.setUserName("神秘人");
            }
        }

        return list;
    }

    /**
     * 获得日期
     *
     * @param date
     * @return dd
     */
    private String getDay(DateTime date) {
        FastDateFormat format = FastDateFormat.getInstance("dd");
        return format.format(date);
    }
}
