package com.starbuds.server.service.user.provider;

import com.google.common.base.Preconditions;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.user.UserMedalTypeEnum;
import com.starbuds.server.common.pojo.data.MedalShinyStatus;
import com.starbuds.server.common.pojo.db.UserMedalShinyStatusDO;
import com.starbuds.server.common.pojo.message.im.rtc.UserMedalShinyStatusChangedMsg;
import com.starbuds.server.common.pojo.mq.QueueSendGiftSnapShotMsg;
import com.starbuds.server.common.service.media.GiftService;
import com.starbuds.server.common.service.message.biz.UserMedalMsgService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.service.user.dao.UserMedalShinyStatusDAO;
import com.starbuds.server.service.user.provider.impl.UserMedalShinyDate;
import com.starbuds.server.service.user.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * 用户勋章状态
 *
 * @author aoe
 * @date 2022/1/14
 */
@Component
@Slf4j
public class UserMedalStatusProvider {

    @Reference(version = "1.0.0")
    private GiftService giftService;

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private UserMedalShinyStatusProvider userMedalShinyStatusProvider;

    @Reference(version = "1.0.0")
    private UserMedalMsgService userMedalMsgService;

    @Resource
    private RtcUserService rtcUserService;

    @Resource
    private UserMedalShinyStatusDAO userMedalShinyStatusDAO;

    @Resource
    private UserMedalShinyDate userMedalShinyDate;

    public void updateMedalStatusToShiny(QueueSendGiftSnapShotMsg msg) {
        Long giftId = msg.getGiftId();
        Preconditions.checkNotNull(giftId, "QueueSendGiftSnapShotMsg 参数错误： giftId is null");
        Long userId = msg.getSenderUserId();
        Preconditions.checkNotNull(userId, "QueueSendGiftSnapShotMsg 参数错误： userId is null");

        Optional<Integer> shinyStatusOpt = getShinyStatusByCache(userId);
        if (shinyStatusOpt.isPresent() && isShinyStatus(shinyStatusOpt.get())) {
            return;
        }

        long wealthValue = giftService.getWealthValue(giftId);
        if (wealthValue < 1) {
            return;
        }
        updateMedalStatusToShiny(userId, UserMedalTypeEnum.WEALTH_LEVEL);
        cleanUserCacheInRoom(userId);
    }

    /**
     * 勋章是否闪亮状态
     *
     * @param userId
     * @return true：闪亮状态
     */
    public boolean isShiny(long userId) {
        Optional<Integer> shinyStatusOpt = getShinyStatusByCache(userId);
        if (shinyStatusOpt.isPresent()) {
            return isShinyStatus(shinyStatusOpt.get());
        }

        int shinyStatus = BoolIntVal.False;
        shinyStatusOpt = getShinyStatusByDB(userId);
        if (shinyStatusOpt.isPresent()) {
            shinyStatus = shinyStatusOpt.get();
        }
        saveTodayMedalShinyStatusToCache(userId, shinyStatus);
        return isShinyStatus(shinyStatus);
    }

    /**
     * 发送勋章闪亮状态改变消息
     *
     * @param userId
     * @param status
     * @return
     */
    public boolean sendChangeMedalShinyStatusMsg(long userId, int status) {
        UserMedalShinyStatusChangedMsg msg = new UserMedalShinyStatusChangedMsg();
        msg.setUserId(userId);

        MedalShinyStatus medalShinyStatus = new MedalShinyStatus();
        medalShinyStatus.setWealthLevel(status);
        msg.setMedalShinyStatus(medalShinyStatus);
        return userMedalMsgService.sendMedalShinyStatusChangedMsg(msg);
    }

    public void cleanUserCacheInRoom(Long userId) {
        Long roomId = rtcUserService.getUserStayRoomId(userId);
        if (roomId == null) {
            return;
        }
        Long delResult = redisUtil.hDelete(RedisRtcKeys.getRtcRoomUser(roomId), String.valueOf(userId));
        log.info("勋章状态改变，清空用户在房间中的缓存 userId={} delResult={}", userId, delResult);
    }

    /**
     * 更新勋章为闪亮状态
     *
     * @param userId
     * @param userMedalType
     */
    private void updateMedalStatusToShiny(long userId, UserMedalTypeEnum userMedalType) {
        UserMedalShinyStatusDO save = userMedalShinyStatusProvider.saveShinyStatus(userId, userMedalType);
        if (save == null) {
            log.error("更新勋章为闪亮状态失败 userId=" + userId);
            return;
        }
        saveTodayMedalShinyStatusToCache(userId, BoolIntVal.True);
        sendChangeMedalShinyStatusMsg(userId, BoolIntVal.True);
    }

    private void saveTodayMedalShinyStatusToCache(long userId, int shinyStatus) {
        String key = RedisUserKeys.getTodayMedalShinyStatus(userId);
        String value = shinyStatus + "";
        redisUtil.setEx(key, value, 1, TimeUnit.DAYS);
        log.info("存储勋章闪亮状态到缓存 key={} value={}", key, value);
    }

    private Optional<Integer> getShinyStatusByCache(long userId) {
        String key = RedisUserKeys.getTodayMedalShinyStatus(userId);
        String value = redisUtil.get(key);
        if (isDev()) {
            log.info("查询勋章闪亮状态2 key={} value={}", key, value);
        }
        if (StringUtils.isBlank(value)) {
            return Optional.empty();
        }
        int status = NumberUtils.toInt(value, BoolIntVal.False);
        return Optional.of(status);
    }

    private Optional<Integer> getShinyStatusByDB(long userId) {
        Integer shinyEndDate = userMedalShinyStatusDAO.selectShinyEndDate(userId);
        if (isDev()) {
            log.info("查询勋章闪亮状态3 userId={} shinyEndDate={}", userId, shinyEndDate);
        }
        if (shinyEndDate == null) {
            return Optional.empty();
        }
        int shinyStatus = BoolIntVal.True;
        LocalDate now = LocalDate.now();
        boolean greaterThanShinyEndDate = userMedalShinyDate.greaterThanShinyEndDate(now, shinyEndDate);
        if (greaterThanShinyEndDate) {
            shinyStatus = BoolIntVal.False;
        }
        return Optional.of(shinyStatus);
    }

    private boolean isShinyStatus(int status) {
        return BoolIntVal.True == status;
    }
}
