package com.starbuds.server.service.user.provider;

import cn.hutool.core.convert.Convert;
import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.user.UserInterviewReadStateEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.UserInterviewHisDO;
import com.starbuds.server.common.pojo.db.UserVisitHisDO;
import com.starbuds.server.common.pojo.message.im.user.UserInterviewMsg;
import com.starbuds.server.common.pojo.user.IntervieweeVO;
import com.starbuds.server.common.pojo.user.IntervieweeVOV2;
import com.starbuds.server.common.pojo.user.VisitorVO;
import com.starbuds.server.common.pojo.user.VisitorVOV2;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.user.dao.UserInterviewHisDAO;
import com.starbuds.server.service.user.dao.UserVisitHisDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import static com.starbuds.server.service.user.util.SpringContextUtil.isDev;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/1/20
 */
@Component
@Slf4j
public class VisitProvider {

    @Autowired
    UserVisitHisDAO userVisitHisDAO;

    @Autowired
    UserInterviewHisDAO userInterviewHisDAO;

    @Reference(version = "1.0.0")
    ImService imService;

    private static String getHisId(Long userId, Long targetUserId, Integer hisDate) {
        return SecureUtil.md5(String.format("%d:%d:%d", userId, targetUserId, hisDate));
    }

    private static Integer getHisDate(Long time) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        return Convert.toInt(sdf.format(new Date(time)));
    }

    /**
     * 创建访问记录
     *
     * @param userId
     * @param targetUserId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean addVisitHis(Long userId, Long targetUserId) {
        // 访问自己忽略
        if (userId.equals(targetUserId)) {
            return false;
        }

        Long curTime = System.currentTimeMillis();
        Integer hisDate = getHisDate(curTime);

        try {
            // 创建访问者记录
            String visitHisId = getHisId(userId, targetUserId, hisDate);
            UserVisitHisDO userVisitHisDO = new UserVisitHisDO();
            userVisitHisDO.setHisId(visitHisId);
            userVisitHisDO.setHisDate(hisDate);
            userVisitHisDO.setUserId(userId);
            userVisitHisDO.setTargetUserId(targetUserId);
            userVisitHisDO.setHisCreateTime(curTime);
            userVisitHisDO.setHisUpdateTime(curTime);
            userVisitHisDO.setStatus(CommonStatus.Normal);
            userVisitHisDO.setHisTimes(1);
            userVisitHisDAO.insertOrUpdate(userVisitHisDO);

            // 创建受访者记录
            String interviewHisId = getHisId(targetUserId, userId, hisDate);
            UserInterviewHisDO userInterviewHisDO = new UserInterviewHisDO();
            userInterviewHisDO.setHisId(interviewHisId);
            userInterviewHisDO.setHisDate(hisDate);
            userInterviewHisDO.setUserId(targetUserId);
            userInterviewHisDO.setVisitorUserId(userId);
            userInterviewHisDO.setHisCreateTime(curTime);
            userInterviewHisDO.setHisUpdateTime(curTime);

            userInterviewHisDO.setStatus(UserInterviewReadStateEnum.UNREAD.getValue());
            userInterviewHisDO.setHisTimes(1);
            userInterviewHisDAO.insertOrUpdate(userInterviewHisDO);

            // 发送访客未读数 消息
            Long unReadCount = userInterviewHisDAO.selectUnReadByUser(userId);
            UserInterviewMsg msg = new UserInterviewMsg();
            msg.setUnReadCount(unReadCount);
            msg.setUserId(userId);
            imService.sendRtcRoomMsg(ImTargetType.USER, null, userId, null, msg);

            return true;
        } catch (Exception e) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            log.error("访问记录创建失败", e);
        }
        return false;
    }

    /**
     * 获取访客列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public PageList<VisitorVO> getVisitorList(Long userId, int pageIndex, int pageSize) {
        PageList<VisitorVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(userVisitHisDAO.selectAll(pageList.getPageCursor(), pageSize, userId));
        pageList.setTotal(userVisitHisDAO.count(userId));
        return pageList;
    }

    /**
     * 获取我的足迹列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public PageList<IntervieweeVO> getIntervieweeList(Long userId, int pageIndex, int pageSize) {
        PageList<IntervieweeVO> pageList = new PageList<>(pageIndex, pageSize);
        pageList.setList(userInterviewHisDAO.selectAll(pageList.getPageCursor(), pageSize, userId));
        pageList.setTotal(userInterviewHisDAO.count(userId));
        return pageList;
    }

    /**
     * 获取访客列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public PageList<VisitorVOV2> getVisitorListV2(Long userId, int pageIndex, int pageSize) {
        PageList<VisitorVOV2> pageList = new PageList<>(pageIndex, pageSize);

        // 最多查询最近30天的访客记录
        Integer dateInt = Integer.valueOf(LocalDate.now().minusDays(30).format(DateTimeFormatter.BASIC_ISO_DATE));

        pageList.setList(userVisitHisDAO.selectAllV2(pageList.getPageCursor(), pageSize, userId, dateInt));
        pageList.setTotal(userVisitHisDAO.selectAllV2Count(userId, dateInt));
        return pageList;
    }

    /**
     * 最近看过的用户id列表
     *
     * @param visitorUserId
     * @param limit
     * @return
     */
    public List<Long> getRecentlySeenUserIds(Long visitorUserId, int limit) {
        List<Long> userIds = userInterviewHisDAO.selectRecentlySeenUserIds(visitorUserId, limit);
        if (CollectionUtils.isEmpty(userIds)) {
            return Collections.emptyList();
        }
        return userIds;
    }

    /**
     * 获取访客次数
     *
     * @param userId
     * @return
     */
    public Integer getVisitorCount(Long userId, Long targetUserId, Integer dateInt) {
        return Optional.ofNullable(userVisitHisDAO.selectVisitorCount(userId, targetUserId, dateInt)).orElse(0);
    }

    /**
     * 谁看过我次数
     *
     * @param userId
     * @return
     */
    public Integer getInterviewCount(Long userId, Long visitorUserId, Integer dateInt) {
        return Optional.ofNullable(userInterviewHisDAO.selectInterviewCount(userId, visitorUserId, dateInt)).orElse(0);
    }

    /**
     * 获取访客次数
     *
     * @param userId
     * @return
     */
    public Long getInterviewCountByUser(Long userId, Integer dateInt) {
        return Optional.ofNullable(userInterviewHisDAO.selectInterviewCountByUser(userId, dateInt)).orElse(0L);
    }

    /**
     * 获取访客次数
     *
     * @param userId
     * @return
     */
    public Long getIntervieweeTotalByUser(Long userId) {
        return Optional.ofNullable(userInterviewHisDAO.selectIntervieweeTotalByUser(userId)).orElse(0L);
    }

    /**
     * 获取足迹次数
     *
     * @param userId
     * @return
     */
    public Long getVisitorCountByUser(Long userId, Integer dateInt) {
        return Optional.ofNullable(userVisitHisDAO.selectVisitorCountByUser(userId, dateInt)).orElse(0L);
    }

    /**
     * 获取足迹次数
     *
     * @param userId
     * @return
     */
    public Long getVisitorTotalByUser(Long userId) {
        return Optional.ofNullable(userVisitHisDAO.selectVisitorTotalCountByUser(userId)).orElse(0L);
    }

    /**
     * 已读谁看过我
     *
     * @param userId
     * @return
     */
    public void resetInterviewCount(Long userId) {
        userInterviewHisDAO.updateReadByUser(userId);
    }

    /**
     * 未读访客数
     *
     * @param userId
     * @return
     */
    public Long getUnReadInterviewCount(Long userId) {
        return userInterviewHisDAO.selectUnReadByUser(userId);
    }

    /**
     * 获取我的足迹列表
     *
     * @param userId
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public PageList<IntervieweeVOV2> getIntervieweeListV2(Long userId, int pageIndex, int pageSize) {
        PageList<IntervieweeVOV2> pageList = new PageList<>(pageIndex, pageSize);

        // 最多查询最近30天的访客记录
        Integer dateInt = Integer.valueOf(LocalDate.now().minusDays(30).format(DateTimeFormatter.BASIC_ISO_DATE));

        pageList.setList(userInterviewHisDAO.selectAllV2(pageList.getPageCursor(), pageSize, userId, dateInt));
        pageList.setTotal(userInterviewHisDAO.selectAllV2Count(userId, dateInt));
        return pageList;
    }
}
