package com.starbuds.server.service.user.rocket.handler;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.SwitchStatusEnum;
import com.starbuds.server.common.constant.user.SwitchTypeEnum;
import com.starbuds.server.common.pojo.db.NiceNoPoolDO;
import com.starbuds.server.common.pojo.db.UserNobleDO;
import com.starbuds.server.common.pojo.mq.QueueNiceNoExpireMsg;
import com.starbuds.server.common.pojo.mq.QueueNobleExpireMsg;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.user.NobleService;
import com.starbuds.server.service.user.dao.*;
import com.starbuds.server.service.user.provider.LiveUserProvider;
import com.starbuds.server.service.user.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * 延时消息处理
 *
 * @author Lay
 * @date 2020/3/2
 */
@Component
@Slf4j
public class DelayMsgHandler {

    @Reference(version = "1.0.0")
    NobleService nobleService;

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    NiceNoPoolDAO niceNoPoolDAO;

    @Resource
    NiceNoHisDAO niceNoHisDAO;

    @Resource
    UserDAO userDAO;

    @Resource
    UserNobleDAO userNobleDAO;

    @Resource
    UserNobleHisDAO userNobleHisDAO;

    @Resource
    LiveUserProvider liveUserProvider;

    @Resource
    UserSwitchDAO userSwitchDAO;

    /**
     * 靓号过期处理
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleNiceNoExpire(QueueNiceNoExpireMsg msg) {
        // 判断靓号的最新记录对应情况
        Long lastHisId = niceNoHisDAO.selectLastId(msg.getNiceNo());
        if (lastHisId == null || !lastHisId.equals(msg.getHisId())) {
            return;
        }

        // 检查是否到期
        NiceNoPoolDO niceNoItem = niceNoPoolDAO.selectById(msg.getNiceNo());
        if (niceNoItem == null || !msg.getUserId().equals(niceNoItem.getUserId())) {
            return;
        }

        Long curTime = System.currentTimeMillis();
        Long bindExpireTime = Optional.ofNullable(niceNoItem.getBindExpireTime()).orElse(-1L);
        if (bindExpireTime > 0) {
            if (System.currentTimeMillis() > bindExpireTime) {
                // 解除绑定
                if (BoolIntVal.True == niceNoItem.getBindStatus()) {
                    // 更新用户
                    userDAO.updateNiceNo(msg.getUserId(), null);

                    // 更新靓号
                    niceNoItem.setBindStatus(BoolIntVal.False);
                    niceNoItem.setBindExpireTime(null);
                    niceNoItem.setUpdateTime(curTime);
                    niceNoItem.setUserId(null);
                    niceNoPoolDAO.updateBind(niceNoItem);

                    // 更新 es
                    userSearchService.setNiceNo(msg.getUserId(), null);
                }
            } else {
                // 重新投递
                msg.setExpireTime(bindExpireTime);
                // 超时60秒后处理
                long startDeliveryTime = bindExpireTime - curTime > TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : bindExpireTime;
                rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NICE_NO_EXPIRE, msg.serialize(), startDeliveryTime);
            }
        }
    }

    /**
     * 贵族过期消息
     *
     * @param msg
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleNobleExpire(QueueNobleExpireMsg msg) {
        // 检查是否过期
        UserNobleDO userNoble = userNobleDAO.selectById(msg.getUserId(), null);
        if (userNoble != null) {
            Long userId = msg.getUserId();
            // 判断记录是否有变化
            Long curHisId = userNobleHisDAO.selectLastId(userId);
            if (!msg.getHisId().equals(curHisId)) {
                return;
            }

            if (userNoble.getExpireTime() > 0) {
                Long curTime = System.currentTimeMillis();
                if (curTime > userNoble.getExpireTime()) {
                    // 过期了 判断是否自动续费
                    if (BoolIntVal.True == userNoble.getAutoRenewal()) {
                        // 尝试自动续费
                        Long balance = nobleService.renewNoble(userId, true, 1, userNoble.getChannelId());
                        if (balance != null && balance >= 0) {
                            // 自动续费成功
                            return;
                        }
                    }

                    // 过期处理
                    userNoble.setStatus(CommonStatus.Disabled);
                    userNoble.setUpdateTime(curTime);
                    userNobleDAO.insertOrUpdate(userNoble);

                    // 重置隐藏关注开关
                    String switchId = String.format("%d:%d", userId, SwitchTypeEnum.HIDE_FOLLOWERS.getCode());
                    userSwitchDAO.updateSwitchStatus(switchId, SwitchStatusEnum.CLOSE.getCode());

                    // 刷新直播间资料
                    liveUserProvider.refreshLiveUserProfile(userId);
                } else {
                    // 重新投递
                    Long expireTime = userNoble.getExpireTime();
                    long startDeliveryTime = expireTime - curTime > TimeConstants.DAY_30_MILLS ? curTime + TimeConstants.DAY_30_MILLS : expireTime;
                    rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.NOBLE_EXPIRE, msg.serialize(), startDeliveryTime);
                }
            }
        }
    }
}
